@@ -1,13 +1,13 @@
 /*
 
-Copyright (C) 2003 John W. Eaton
+Copyright (C) 2003, 2004, 2005, 2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +15,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -38,9 +37,9 @@ public:
 
   charNDArray (void) : MArrayN<char> () { }
 
-  charNDArray (dim_vector& dv) : MArrayN<char> (dv) { }
+  charNDArray (const dim_vector& dv) : MArrayN<char> (dv) { }
 
-  charNDArray (dim_vector& dv, char val) : MArrayN<char> (dv, val) { }
+  charNDArray (const dim_vector& dv, char val) : MArrayN<char> (dv, val) { }
   
   charNDArray (const charNDArray& a) : MArrayN<char> (a) { }
 
@@ -95,14 +94,14 @@ private:
   charNDArray (char *d, dim_vector& dv) : MArrayN<char> (d, dv) { }
 };
 
-NDS_CMP_OP_DECLS (charNDArray, char)
-NDS_BOOL_OP_DECLS (charNDArray, char)
+NDS_CMP_OP_DECLS (charNDArray, char, OCTAVE_API)
+NDS_BOOL_OP_DECLS (charNDArray, char, OCTAVE_API)
 
-SND_CMP_OP_DECLS (char, charNDArray)
-SND_BOOL_OP_DECLS (char, charNDArray)
+SND_CMP_OP_DECLS (char, charNDArray, OCTAVE_API)
+SND_BOOL_OP_DECLS (char, charNDArray, OCTAVE_API)
 
-NDND_CMP_OP_DECLS (charNDArray, charNDArray)
-NDND_BOOL_OP_DECLS (charNDArray, charNDArray)
+NDND_CMP_OP_DECLS (charNDArray, charNDArray, OCTAVE_API)
+NDND_BOOL_OP_DECLS (charNDArray, charNDArray, OCTAVE_API)
 
 MARRAY_FORWARD_DEFS (MArrayN, charNDArray, char)
 
