@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
+              2005, 2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -41,10 +41,6 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 
 #include "f77-fcn.h"
 
-#ifndef M_LOG10E
-#define M_LOG10E 0.43429448190325182765
-#endif
-
 // double -> double mappers.
 
 double
@@ -100,6 +96,38 @@ signum (double x)
   return xisnan (x) ? octave_NaN : tmp;
 }
 
+double
+xlog2 (double x)
+{
+#if defined (HAVE_LOG2)
+  return log2 (x);
+#else
+#if defined (M_LN2)
+  static double ln2 = M_LN2;
+#else
+  static double ln2 = log2 (2);
+#endif
+
+  return log (x) / ln2;
+#endif
+}
+
+double
+xexp2 (double x)
+{
+#if defined (HAVE_EXP2)
+  return exp2 (x);
+#else
+#if defined (M_LN2)
+  static double ln2 = M_LN2;
+#else
+  static double ln2 = log2 (2);
+#endif
+
+  return exp (x * ln2);
+#endif
+}
+
 // double -> bool mappers.
 
 bool
@@ -129,12 +157,12 @@ octave_is_NA (double x)
 bool
 octave_is_NaN_or_NA (double x)
 {
-  return lo_ieee_is_NaN_or_NA (x);
+  return lo_ieee_isnan (x);
 }
 
 // (double, double) -> double mappers.
 
-// XXX FIXME XXX -- need to handle NA too?
+// FIXME -- need to handle NA too?
 
 double
 xmin (double x, double y)
@@ -145,9 +173,9 @@ xmin (double x, double y)
   if (y <= x)
     return y;
 
-  if (octave_is_NaN_or_NA (x) && ! octave_is_NaN_or_NA (y))
+  if (xisnan (x) && ! xisnan (y))
     return y;
-  else if (octave_is_NaN_or_NA (y) && ! octave_is_NaN_or_NA (x))
+  else if (xisnan (y) && ! xisnan (x))
     return x;
   else if (octave_is_NA (x) || octave_is_NA (y))
     return octave_NA;
@@ -164,9 +192,9 @@ xmax (double x, double y)
   if (y >= x)
     return y;
 
-  if (octave_is_NaN_or_NA (x) && ! octave_is_NaN_or_NA (y))
+  if (xisnan (x) && ! xisnan (y))
     return y;
-  else if (octave_is_NaN_or_NA (y) && ! octave_is_NaN_or_NA (x))
+  else if (xisnan (y) && ! xisnan (x))
     return x;
   else if (octave_is_NA (x) || octave_is_NA (y))
     return octave_NA;
@@ -181,7 +209,7 @@ acos (const Complex& x)
 {
   static Complex i (0, 1);
 
-  return (real (x) * imag (x) < 0.0) ? i * acosh (x) : -i * acosh (x);
+  return -i * (log (x + i * (sqrt (1.0 - x*x))));
 }
 
 Complex
@@ -245,7 +273,9 @@ xround (const Complex& x)
 Complex
 signum (const Complex& x)
 {
-  return x / abs (x);
+  double tmp = abs (x);
+
+  return tmp == 0 ? 0.0 : x / tmp;
 }
 
 // complex -> bool mappers.
@@ -262,8 +292,8 @@ xfinite (const Complex& x)
   double rx = real (x);
   double ix = imag (x);
 
-  return (xfinite (rx) && ! octave_is_NaN_or_NA (rx)
-	  && xfinite (ix) && ! octave_is_NaN_or_NA (ix));
+  return (xfinite (rx) && ! xisnan (rx)
+	  && xfinite (ix) && ! xisnan (ix));
 }
 
 bool
@@ -281,12 +311,12 @@ octave_is_NA (const Complex& x)
 bool
 octave_is_NaN_or_NA (const Complex& x)
 {
-  return (octave_is_NaN_or_NA (real (x)) || octave_is_NaN_or_NA (imag (x)));
+  return (xisnan (real (x)) || xisnan (imag (x)));
 }
 
 // (complex, complex) -> complex mappers.
 
-// XXX FIXME XXX -- need to handle NA too?
+// FIXME -- need to handle NA too?
 
 Complex
 xmin (const Complex& x, const Complex& y)
