@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996 John W. Eaton
+Copyright (C) 1996, 1998, 2002, 2003, 2004, 2005, 2006, 2007
+              John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -37,6 +37,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "lo-ieee.h"
 #include "lo-specfun.h"
 #include "mx-inlines.cc"
+#include "lo-mappers.h"
 
 #ifndef M_PI
 #define M_PI 3.14159265358979323846
@@ -159,24 +160,40 @@ erfc (double x)
 double
 xgamma (double x)
 {
+#if defined (HAVE_TGAMMA)
+  return tgamma (x);
+#else
   double result;
-  F77_XFCN (xdgamma, XDGAMMA, (x, result));
+
+  if (xisnan (x))
+    result = x;
+  else if ((x <= 0 && D_NINT (x) == x) || xisinf (x))
+    result = octave_Inf;
+  else
+    F77_XFCN (xdgamma, XDGAMMA, (x, result));
+
   return result;
+#endif
 }
 
 double
 xlgamma (double x)
 {
+#if defined (HAVE_LGAMMA)
+  return lgamma (x);
+#else
   double result;
   double sgngam;
 
-  if (x < 0)
-    (*current_liboctave_error_handler)
-      ("xlgamma: argument must be nonnegative");
-
-  F77_XFCN (dlgams, DLGAMS, (x, result, sgngam));
+  if (xisnan (x))
+    result = x;
+  else if (xisinf (x))
+    result = octave_Inf;
+  else
+    F77_XFCN (dlgams, DLGAMS, (x, result, sgngam));
 
   return result;
+#endif
 }
 
 static inline Complex
@@ -947,7 +964,7 @@ double
 betainc (double x, double a, double b)
 {
   double retval;
-  F77_FUNC (xdbetai, XDBETAI) (x, a, b, retval);
+  F77_XFCN (xdbetai, XDBETAI, (x, a, b, retval));
   return retval;
 }
 
@@ -1226,7 +1243,7 @@ betainc (const NDArray& x, const NDArray& a, const NDArray& b)
   return retval;
 }
 
-// XXX FIXME XXX -- there is still room for improvement here...
+// FIXME -- there is still room for improvement here...
 
 double
 gammainc (double x, double a, bool& err)
