@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1996, 1997, 1998, 2000, 2001, 2003, 2004, 2005, 2006,
+              2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -26,31 +26,31 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 
 #include "mx-inlines.cc"
 
-#define BIN_OP_DECL(R, OP, X, Y) \
-  extern OCTAVE_API R OP (const X&, const Y&)
+#define BIN_OP_DECL(R, OP, X, Y, API) \
+  extern API R OP (const X&, const Y&)
 
 class boolMatrix;
 class boolNDArray;
 
-#define CMP_OP_DECL(OP, X, Y) \
-  extern OCTAVE_API boolMatrix OP (const X&, const Y&)
+#define CMP_OP_DECL(OP, X, Y, API) \
+  extern API boolMatrix OP (const X&, const Y&)
 
-#define NDCMP_OP_DECL(OP, X, Y) \
-  extern OCTAVE_API boolNDArray OP (const X&, const Y&)
+#define NDCMP_OP_DECL(OP, X, Y, API) \
+  extern API boolNDArray OP (const X&, const Y&)
 
-#define BOOL_OP_DECL(OP, X, Y) \
-  extern OCTAVE_API boolMatrix OP (const X&, const Y&)
+#define BOOL_OP_DECL(OP, X, Y, API) \
+  extern API boolMatrix OP (const X&, const Y&)
 
-#define NDBOOL_OP_DECL(OP, X, Y) \
-  extern OCTAVE_API boolNDArray OP (const X&, const Y&)
+#define NDBOOL_OP_DECL(OP, X, Y, API) \
+  extern API boolNDArray OP (const X&, const Y&)
 
 // vector by scalar operations.
 
-#define VS_BIN_OP_DECLS(R, V, S) \
-  BIN_OP_DECL (R, operator +, V, S); \
-  BIN_OP_DECL (R, operator -, V, S); \
-  BIN_OP_DECL (R, operator *, V, S); \
-  BIN_OP_DECL (R, operator /, V, S);
+#define VS_BIN_OP_DECLS(R, V, S, API) \
+  BIN_OP_DECL (R, operator +, V, S, API); \
+  BIN_OP_DECL (R, operator -, V, S, API); \
+  BIN_OP_DECL (R, operator *, V, S, API); \
+  BIN_OP_DECL (R, operator /, V, S, API);
 
 #define VS_BIN_OP(R, F, OP, V, S) \
   R \
@@ -72,16 +72,16 @@ class boolNDArray;
   VS_BIN_OP (R, operator *, *, V, S) \
   VS_BIN_OP (R, operator /, /, V, S)
 
-#define VS_OP_DECLS(R, V, S) \
-  VS_BIN_OP_DECLS(R, V, S)
+#define VS_OP_DECLS(R, V, S, API) \
+  VS_BIN_OP_DECLS(R, V, S, API)
 
 // scalar by vector by operations.
 
-#define SV_BIN_OP_DECLS(R, S, V) \
-  BIN_OP_DECL (R, operator +, S, V); \
-  BIN_OP_DECL (R, operator -, S, V); \
-  BIN_OP_DECL (R, operator *, S, V); \
-  BIN_OP_DECL (R, operator /, S, V);
+#define SV_BIN_OP_DECLS(R, S, V, API) \
+  BIN_OP_DECL (R, operator +, S, V, API); \
+  BIN_OP_DECL (R, operator -, S, V, API); \
+  BIN_OP_DECL (R, operator *, S, V, API); \
+  BIN_OP_DECL (R, operator /, S, V, API);
 
 #define SV_BIN_OP(R, F, OP, S, V) \
   R \
@@ -103,16 +103,16 @@ class boolNDArray;
   SV_BIN_OP (R, operator *, *, S, V) \
   SV_BIN_OP (R, operator /, /, S, V)
 
-#define SV_OP_DECLS(R, S, V) \
-  SV_BIN_OP_DECLS(R, S, V)
+#define SV_OP_DECLS(R, S, V, API) \
+  SV_BIN_OP_DECLS(R, S, V, API)
 
 // vector by vector operations.
 
-#define VV_BIN_OP_DECLS(R, V1, V2) \
-  BIN_OP_DECL (R, operator +, V1, V2); \
-  BIN_OP_DECL (R, operator -, V1, V2); \
-  BIN_OP_DECL (R, product,    V1, V2); \
-  BIN_OP_DECL (R, quotient,   V1, V2);
+#define VV_BIN_OP_DECLS(R, V1, V2, API) \
+  BIN_OP_DECL (R, operator +, V1, V2, API); \
+  BIN_OP_DECL (R, operator -, V1, V2, API); \
+  BIN_OP_DECL (R, product,    V1, V2, API); \
+  BIN_OP_DECL (R, quotient,   V1, V2, API);
 
 #define VV_BIN_OP(R, F, OP, V1, V2) \
   R \
@@ -142,16 +142,16 @@ class boolNDArray;
   VV_BIN_OP (R, product,    *, V1, V2) \
   VV_BIN_OP (R, quotient,   /, V1, V2)
 
-#define VV_OP_DECLS(R, V1, V2) \
-  VV_BIN_OP_DECLS(R, V1, V2)
+#define VV_OP_DECLS(R, V1, V2, API) \
+  VV_BIN_OP_DECLS(R, V1, V2, API)
 
 // matrix by scalar operations.
 
-#define MS_BIN_OP_DECLS(R, M, S) \
-  BIN_OP_DECL (R, operator +, M, S); \
-  BIN_OP_DECL (R, operator -, M, S); \
-  BIN_OP_DECL (R, operator *, M, S); \
-  BIN_OP_DECL (R, operator /, M, S);
+#define MS_BIN_OP_DECLS(R, M, S, API) \
+  BIN_OP_DECL (R, operator +, M, S, API); \
+  BIN_OP_DECL (R, operator -, M, S, API); \
+  BIN_OP_DECL (R, operator *, M, S, API); \
+  BIN_OP_DECL (R, operator /, M, S, API);
 
 #define MS_BIN_OP(R, OP, M, S, F) \
   R \
@@ -174,13 +174,13 @@ class boolNDArray;
   MS_BIN_OP (R, operator *, M, S, mx_inline_multiply) \
   MS_BIN_OP (R, operator /, M, S, mx_inline_divide)
 
-#define MS_CMP_OP_DECLS(M, S) \
-  CMP_OP_DECL (mx_el_lt, M, S); \
-  CMP_OP_DECL (mx_el_le, M, S); \
-  CMP_OP_DECL (mx_el_ge, M, S); \
-  CMP_OP_DECL (mx_el_gt, M, S); \
-  CMP_OP_DECL (mx_el_eq, M, S); \
-  CMP_OP_DECL (mx_el_ne, M, S);
+#define MS_CMP_OP_DECLS(M, S, API) \
+  CMP_OP_DECL (mx_el_lt, M, S, API); \
+  CMP_OP_DECL (mx_el_le, M, S, API); \
+  CMP_OP_DECL (mx_el_ge, M, S, API); \
+  CMP_OP_DECL (mx_el_gt, M, S, API); \
+  CMP_OP_DECL (mx_el_eq, M, S, API); \
+  CMP_OP_DECL (mx_el_ne, M, S, API);
 
 #define MS_CMP_OP(F, OP, M, MC, S, SC) \
   boolMatrix \
@@ -211,9 +211,9 @@ class boolNDArray;
   MS_CMP_OP (mx_el_eq, ==, M,   , S,   ) \
   MS_CMP_OP (mx_el_ne, !=, M,   , S,   )
 
-#define MS_BOOL_OP_DECLS(M, S) \
-  BOOL_OP_DECL (mx_el_and, M, S); \
-  BOOL_OP_DECL (mx_el_or,  M, S); \
+#define MS_BOOL_OP_DECLS(M, S, API) \
+  BOOL_OP_DECL (mx_el_and, M, S, API); \
+  BOOL_OP_DECL (mx_el_or,  M, S, API); \
 
 #define MS_BOOL_OP(F, OP, M, S, LHS_ZERO, RHS_ZERO) \
   boolMatrix \
@@ -243,18 +243,18 @@ class boolNDArray;
 #define MS_BOOL_OPS(M, S, ZERO) \
   MS_BOOL_OPS2(M, S, ZERO, ZERO)
 
-#define MS_OP_DECLS(R, M, S) \
-  MS_BIN_OP_DECLS (R, M, S) \
-  MS_CMP_OP_DECLS (M, S) \
-  MS_BOOL_OP_DECLS (M, S) \
+#define MS_OP_DECLS(R, M, S, API) \
+  MS_BIN_OP_DECLS (R, M, S, API) \
+  MS_CMP_OP_DECLS (M, S, API) \
+  MS_BOOL_OP_DECLS (M, S, API) \
 
 // scalar by matrix operations.
 
-#define SM_BIN_OP_DECLS(R, S, M) \
-  BIN_OP_DECL (R, operator +, S, M); \
-  BIN_OP_DECL (R, operator -, S, M); \
-  BIN_OP_DECL (R, operator *, S, M); \
-  BIN_OP_DECL (R, operator /, S, M);
+#define SM_BIN_OP_DECLS(R, S, M, API) \
+  BIN_OP_DECL (R, operator +, S, M, API); \
+  BIN_OP_DECL (R, operator -, S, M, API); \
+  BIN_OP_DECL (R, operator *, S, M, API); \
+  BIN_OP_DECL (R, operator /, S, M, API);
 
 #define SM_BIN_OP(R, OP, S, M, F) \
   R \
@@ -277,13 +277,13 @@ class boolNDArray;
   SM_BIN_OP (R, operator *, S, M, mx_inline_multiply) \
   SM_BIN_OP (R, operator /, S, M, mx_inline_divide)
 
-#define SM_CMP_OP_DECLS(S, M) \
-  CMP_OP_DECL (mx_el_lt, S, M); \
-  CMP_OP_DECL (mx_el_le, S, M); \
-  CMP_OP_DECL (mx_el_ge, S, M); \
-  CMP_OP_DECL (mx_el_gt, S, M); \
-  CMP_OP_DECL (mx_el_eq, S, M); \
-  CMP_OP_DECL (mx_el_ne, S, M);
+#define SM_CMP_OP_DECLS(S, M, API) \
+  CMP_OP_DECL (mx_el_lt, S, M, API); \
+  CMP_OP_DECL (mx_el_le, S, M, API); \
+  CMP_OP_DECL (mx_el_ge, S, M, API); \
+  CMP_OP_DECL (mx_el_gt, S, M, API); \
+  CMP_OP_DECL (mx_el_eq, S, M, API); \
+  CMP_OP_DECL (mx_el_ne, S, M, API);
 
 #define SM_CMP_OP(F, OP, S, SC, M, MC) \
   boolMatrix \
@@ -314,9 +314,9 @@ class boolNDArray;
   SM_CMP_OP (mx_el_eq, ==, S,   , M,   ) \
   SM_CMP_OP (mx_el_ne, !=, S,   , M,   )
 
-#define SM_BOOL_OP_DECLS(S, M) \
-  BOOL_OP_DECL (mx_el_and, S, M); \
-  BOOL_OP_DECL (mx_el_or,  S, M); \
+#define SM_BOOL_OP_DECLS(S, M, API) \
+  BOOL_OP_DECL (mx_el_and, S, M, API); \
+  BOOL_OP_DECL (mx_el_or,  S, M, API); \
 
 #define SM_BOOL_OP(F, OP, S, M, LHS_ZERO, RHS_ZERO) \
   boolMatrix \
@@ -346,18 +346,18 @@ class boolNDArray;
 #define SM_BOOL_OPS(S, M, ZERO) \
   SM_BOOL_OPS2(S, M, ZERO, ZERO)
 
-#define SM_OP_DECLS(R, S, M) \
-  SM_BIN_OP_DECLS (R, S, M) \
-  SM_CMP_OP_DECLS (S, M) \
-  SM_BOOL_OP_DECLS (S, M) \
+#define SM_OP_DECLS(R, S, M, API) \
+  SM_BIN_OP_DECLS (R, S, M, API) \
+  SM_CMP_OP_DECLS (S, M, API) \
+  SM_BOOL_OP_DECLS (S, M, API) \
 
 // matrix by matrix operations.
 
-#define MM_BIN_OP_DECLS(R, M1, M2) \
-  BIN_OP_DECL (R, operator +, M1, M2); \
-  BIN_OP_DECL (R, operator -, M1, M2); \
-  BIN_OP_DECL (R, product,    M1, M2); \
-  BIN_OP_DECL (R, quotient,   M1, M2);
+#define MM_BIN_OP_DECLS(R, M1, M2, API) \
+  BIN_OP_DECL (R, operator +, M1, M2, API); \
+  BIN_OP_DECL (R, operator -, M1, M2, API); \
+  BIN_OP_DECL (R, product,    M1, M2, API); \
+  BIN_OP_DECL (R, quotient,   M1, M2, API);
 
 #define MM_BIN_OP(R, OP, M1, M2, F) \
   R \
@@ -390,13 +390,13 @@ class boolNDArray;
   MM_BIN_OP (R, product,    M1, M2, mx_inline_multiply) \
   MM_BIN_OP (R, quotient,   M1, M2, mx_inline_divide)
 
-#define MM_CMP_OP_DECLS(M1, M2) \
-  CMP_OP_DECL (mx_el_lt, M1, M2); \
-  CMP_OP_DECL (mx_el_le, M1, M2); \
-  CMP_OP_DECL (mx_el_ge, M1, M2); \
-  CMP_OP_DECL (mx_el_gt, M1, M2); \
-  CMP_OP_DECL (mx_el_eq, M1, M2); \
-  CMP_OP_DECL (mx_el_ne, M1, M2);
+#define MM_CMP_OP_DECLS(M1, M2, API) \
+  CMP_OP_DECL (mx_el_lt, M1, M2, API); \
+  CMP_OP_DECL (mx_el_le, M1, M2, API); \
+  CMP_OP_DECL (mx_el_ge, M1, M2, API); \
+  CMP_OP_DECL (mx_el_gt, M1, M2, API); \
+  CMP_OP_DECL (mx_el_eq, M1, M2, API); \
+  CMP_OP_DECL (mx_el_ne, M1, M2, API);
 
 #define MM_CMP_OP(F, OP, M1, C1, M2, C2) \
   boolMatrix \
@@ -432,9 +432,9 @@ class boolNDArray;
   MM_CMP_OP (mx_el_eq, ==, M1,   , M2,   ) \
   MM_CMP_OP (mx_el_ne, !=, M1,   , M2,   )
 
-#define MM_BOOL_OP_DECLS(M1, M2) \
-  BOOL_OP_DECL (mx_el_and, M1, M2); \
-  BOOL_OP_DECL (mx_el_or,  M1, M2);
+#define MM_BOOL_OP_DECLS(M1, M2, API) \
+  BOOL_OP_DECL (mx_el_and, M1, M2, API); \
+  BOOL_OP_DECL (mx_el_or,  M1, M2, API);
 
 #define MM_BOOL_OP(F, OP, M1, M2, LHS_ZERO, RHS_ZERO) \
   boolMatrix \
@@ -476,18 +476,18 @@ class boolNDArray;
 #define MM_BOOL_OPS(M1, M2, ZERO) \
   MM_BOOL_OPS2(M1, M2, ZERO, ZERO)
 
-#define MM_OP_DECLS(R, M1, M2) \
-  MM_BIN_OP_DECLS (R, M1, M2) \
-  MM_CMP_OP_DECLS (M1, M2) \
-  MM_BOOL_OP_DECLS (M1, M2)
+#define MM_OP_DECLS(R, M1, M2, API) \
+  MM_BIN_OP_DECLS (R, M1, M2, API) \
+  MM_CMP_OP_DECLS (M1, M2, API) \
+  MM_BOOL_OP_DECLS (M1, M2, API)
 
 // N-d matrix by scalar operations.
 
-#define NDS_BIN_OP_DECLS(R, ND, S) \
-  BIN_OP_DECL (R, operator +, ND, S); \
-  BIN_OP_DECL (R, operator -, ND, S); \
-  BIN_OP_DECL (R, operator *, ND, S); \
-  BIN_OP_DECL (R, operator /, ND, S);
+#define NDS_BIN_OP_DECLS(R, ND, S, API) \
+  BIN_OP_DECL (R, operator +, ND, S, API); \
+  BIN_OP_DECL (R, operator -, ND, S, API); \
+  BIN_OP_DECL (R, operator *, ND, S, API); \
+  BIN_OP_DECL (R, operator /, ND, S, API);
 
 #define NDS_BIN_OP(R, OP, ND, S, F) \
   R \
@@ -509,13 +509,13 @@ class boolNDArray;
   NDS_BIN_OP (R, operator *, ND, S, mx_inline_multiply) \
   NDS_BIN_OP (R, operator /, ND, S, mx_inline_divide)
 
-#define NDS_CMP_OP_DECLS(ND, S) \
-  NDCMP_OP_DECL (mx_el_lt, ND, S); \
-  NDCMP_OP_DECL (mx_el_le, ND, S); \
-  NDCMP_OP_DECL (mx_el_ge, ND, S); \
-  NDCMP_OP_DECL (mx_el_gt, ND, S); \
-  NDCMP_OP_DECL (mx_el_eq, ND, S); \
-  NDCMP_OP_DECL (mx_el_ne, ND, S);
+#define NDS_CMP_OP_DECLS(ND, S, API) \
+  NDCMP_OP_DECL (mx_el_lt, ND, S, API); \
+  NDCMP_OP_DECL (mx_el_le, ND, S, API); \
+  NDCMP_OP_DECL (mx_el_ge, ND, S, API); \
+  NDCMP_OP_DECL (mx_el_gt, ND, S, API); \
+  NDCMP_OP_DECL (mx_el_eq, ND, S, API); \
+  NDCMP_OP_DECL (mx_el_ne, ND, S, API);
 
 #define NDS_CMP_OP(F, OP, ND, NDC, S, SC) \
   boolNDArray \
@@ -589,9 +589,9 @@ class boolNDArray;
   NDS_CMP_OP2 (mx_el_eq, ==, ND,    , S,   , SPEC1, SPEC2) \
   NDS_CMP_OP2 (mx_el_ne, !=, ND,    , S,   , SPEC1, SPEC2)
 
-#define NDS_BOOL_OP_DECLS(ND, S) \
-  NDBOOL_OP_DECL (mx_el_and, ND, S); \
-  NDBOOL_OP_DECL (mx_el_or,  ND, S);
+#define NDS_BOOL_OP_DECLS(ND, S, API) \
+  NDBOOL_OP_DECL (mx_el_and, ND, S, API); \
+  NDBOOL_OP_DECL (mx_el_or,  ND, S, API);
 
 #define NDS_BOOL_OP(F, OP, ND, S, LHS_ZERO, RHS_ZERO) \
   boolNDArray \
@@ -619,18 +619,18 @@ class boolNDArray;
 #define NDS_BOOL_OPS(ND, S, ZERO) \
   NDS_BOOL_OPS2(ND, S, ZERO, ZERO)
 
-#define NDS_OP_DECLS(R, ND, S) \
-  NDS_BIN_OP_DECLS (R, ND, S) \
-  NDS_CMP_OP_DECLS (ND, S) \
-  NDS_BOOL_OP_DECLS (ND, S)
+#define NDS_OP_DECLS(R, ND, S, API) \
+  NDS_BIN_OP_DECLS (R, ND, S, API) \
+  NDS_CMP_OP_DECLS (ND, S, API) \
+  NDS_BOOL_OP_DECLS (ND, S, API)
 
 // scalar by N-d matrix operations.
 
-#define SND_BIN_OP_DECLS(R, S, ND) \
-  BIN_OP_DECL (R, operator +, S, ND); \
-  BIN_OP_DECL (R, operator -, S, ND); \
-  BIN_OP_DECL (R, operator *, S, ND); \
-  BIN_OP_DECL (R, operator /, S, ND);
+#define SND_BIN_OP_DECLS(R, S, ND, API) \
+  BIN_OP_DECL (R, operator +, S, ND, API); \
+  BIN_OP_DECL (R, operator -, S, ND, API); \
+  BIN_OP_DECL (R, operator *, S, ND, API); \
+  BIN_OP_DECL (R, operator /, S, ND, API);
 
 #define SND_BIN_OP(R, OP, S, ND, F) \
   R \
@@ -652,13 +652,13 @@ class boolNDArray;
   SND_BIN_OP (R, operator *, S, ND, mx_inline_multiply) \
   SND_BIN_OP (R, operator /, S, ND, mx_inline_divide)
 
-#define SND_CMP_OP_DECLS(S, ND) \
-  NDCMP_OP_DECL (mx_el_lt, S, ND); \
-  NDCMP_OP_DECL (mx_el_le, S, ND); \
-  NDCMP_OP_DECL (mx_el_ge, S, ND); \
-  NDCMP_OP_DECL (mx_el_gt, S, ND); \
-  NDCMP_OP_DECL (mx_el_eq, S, ND); \
-  NDCMP_OP_DECL (mx_el_ne, S, ND);
+#define SND_CMP_OP_DECLS(S, ND, API) \
+  NDCMP_OP_DECL (mx_el_lt, S, ND, API); \
+  NDCMP_OP_DECL (mx_el_le, S, ND, API); \
+  NDCMP_OP_DECL (mx_el_ge, S, ND, API); \
+  NDCMP_OP_DECL (mx_el_gt, S, ND, API); \
+  NDCMP_OP_DECL (mx_el_eq, S, ND, API); \
+  NDCMP_OP_DECL (mx_el_ne, S, ND, API);
 
 #define SND_CMP_OP(F, OP, S, SC, ND, NDC) \
   boolNDArray \
@@ -732,9 +732,9 @@ class boolNDArray;
   SND_CMP_OP2 (mx_el_eq, ==, S,   , ND,    , SPEC1, SPEC2) \
   SND_CMP_OP2 (mx_el_ne, !=, S,   , ND,    , SPEC1, SPEC2)
 
-#define SND_BOOL_OP_DECLS(S, ND) \
-  NDBOOL_OP_DECL (mx_el_and, S, ND); \
-  NDBOOL_OP_DECL (mx_el_or,  S, ND);
+#define SND_BOOL_OP_DECLS(S, ND, API) \
+  NDBOOL_OP_DECL (mx_el_and, S, ND, API); \
+  NDBOOL_OP_DECL (mx_el_or,  S, ND, API);
 
 #define SND_BOOL_OP(F, OP, S, ND, LHS_ZERO, RHS_ZERO) \
   boolNDArray \
@@ -762,18 +762,18 @@ class boolNDArray;
 #define SND_BOOL_OPS(S, ND, ZERO) \
   SND_BOOL_OPS2(S, ND, ZERO, ZERO)
 
-#define SND_OP_DECLS(R, S, ND) \
-  SND_BIN_OP_DECLS (R, S, ND) \
-  SND_CMP_OP_DECLS (S, ND) \
-  SND_BOOL_OP_DECLS (S, ND)
+#define SND_OP_DECLS(R, S, ND, API) \
+  SND_BIN_OP_DECLS (R, S, ND, API) \
+  SND_CMP_OP_DECLS (S, ND, API) \
+  SND_BOOL_OP_DECLS (S, ND, API)
 
 // N-d matrix by N-d matrix operations.
 
-#define NDND_BIN_OP_DECLS(R, ND1, ND2) \
-  BIN_OP_DECL (R, operator +, ND1, ND2); \
-  BIN_OP_DECL (R, operator -, ND1, ND2); \
-  BIN_OP_DECL (R, product,    ND1, ND2); \
-  BIN_OP_DECL (R, quotient,   ND1, ND2);
+#define NDND_BIN_OP_DECLS(R, ND1, ND2, API) \
+  BIN_OP_DECL (R, operator +, ND1, ND2, API); \
+  BIN_OP_DECL (R, operator -, ND1, ND2, API); \
+  BIN_OP_DECL (R, product,    ND1, ND2, API); \
+  BIN_OP_DECL (R, quotient,   ND1, ND2, API);
 
 #define NDND_BIN_OP(R, OP, ND1, ND2, F) \
   R \
@@ -805,13 +805,13 @@ class boolNDArray;
   NDND_BIN_OP (R, product,    ND1, ND2, mx_inline_multiply) \
   NDND_BIN_OP (R, quotient,   ND1, ND2, mx_inline_divide)
 
-#define NDND_CMP_OP_DECLS(ND1, ND2) \
-  NDCMP_OP_DECL (mx_el_lt, ND1, ND2); \
-  NDCMP_OP_DECL (mx_el_le, ND1, ND2); \
-  NDCMP_OP_DECL (mx_el_ge, ND1, ND2); \
-  NDCMP_OP_DECL (mx_el_gt, ND1, ND2); \
-  NDCMP_OP_DECL (mx_el_eq, ND1, ND2); \
-  NDCMP_OP_DECL (mx_el_ne, ND1, ND2);
+#define NDND_CMP_OP_DECLS(ND1, ND2, API) \
+  NDCMP_OP_DECL (mx_el_lt, ND1, ND2, API); \
+  NDCMP_OP_DECL (mx_el_le, ND1, ND2, API); \
+  NDCMP_OP_DECL (mx_el_ge, ND1, ND2, API); \
+  NDCMP_OP_DECL (mx_el_gt, ND1, ND2, API); \
+  NDCMP_OP_DECL (mx_el_eq, ND1, ND2, API); \
+  NDCMP_OP_DECL (mx_el_ne, ND1, ND2, API);
 
 #define NDND_CMP_OP(F, OP, ND1, C1, ND2, C2) \
   boolNDArray \
@@ -843,9 +843,9 @@ class boolNDArray;
   NDND_CMP_OP (mx_el_eq, ==, ND1,   , ND2,   ) \
   NDND_CMP_OP (mx_el_ne, !=, ND1,   , ND2,   )
 
-#define NDND_BOOL_OP_DECLS(ND1, ND2) \
-  NDBOOL_OP_DECL (mx_el_and, ND1, ND2); \
-  NDBOOL_OP_DECL (mx_el_or,  ND1, ND2);
+#define NDND_BOOL_OP_DECLS(ND1, ND2, API) \
+  NDBOOL_OP_DECL (mx_el_and, ND1, ND2, API); \
+  NDBOOL_OP_DECL (mx_el_or,  ND1, ND2, API);
 
 #define NDND_BOOL_OP(F, OP, ND1, ND2, LHS_ZERO, RHS_ZERO) \
   boolNDArray \
@@ -879,16 +879,16 @@ class boolNDArray;
 #define NDND_BOOL_OPS(ND1, ND2, ZERO) \
   NDND_BOOL_OPS2(ND1, ND2, ZERO, ZERO)
 
-#define NDND_OP_DECLS(R, ND1, ND2) \
-  NDND_BIN_OP_DECLS (R, ND1, ND2) \
-  NDND_CMP_OP_DECLS (ND1, ND2) \
-  NDND_BOOL_OP_DECLS (ND1, ND2)
+#define NDND_OP_DECLS(R, ND1, ND2, API) \
+  NDND_BIN_OP_DECLS (R, ND1, ND2, API) \
+  NDND_CMP_OP_DECLS (ND1, ND2, API) \
+  NDND_BOOL_OP_DECLS (ND1, ND2, API)
 
 // scalar by diagonal matrix operations.
 
-#define SDM_BIN_OP_DECLS(R, S, DM) \
-  BIN_OP_DECL (R, operator +, S, DM); \
-  BIN_OP_DECL (R, operator -, S, DM);
+#define SDM_BIN_OP_DECLS(R, S, DM, API) \
+  BIN_OP_DECL (R, operator +, S, DM, API); \
+  BIN_OP_DECL (R, operator -, S, DM, API);
 
 #define SDM_BIN_OP(R, OP, S, DM, OPEQ) \
   R \
@@ -909,14 +909,14 @@ class boolNDArray;
   SDM_BIN_OP (R, operator +, S, DM, +=) \
   SDM_BIN_OP (R, operator -, S, DM, -=)
 
-#define SDM_OP_DECLS(R, S, DM) \
-  SDM_BIN_OP_DECLS(R, S, DM)
+#define SDM_OP_DECLS(R, S, DM, API) \
+  SDM_BIN_OP_DECLS(R, S, DM, API)
 
 // diagonal matrix by scalar operations.
 
-#define DMS_BIN_OP_DECLS(R, DM, S) \
-  BIN_OP_DECL (R, operator +, DM, S); \
-  BIN_OP_DECL (R, operator -, DM, S);
+#define DMS_BIN_OP_DECLS(R, DM, S, API) \
+  BIN_OP_DECL (R, operator +, DM, S, API); \
+  BIN_OP_DECL (R, operator -, DM, S, API);
 
 #define DMS_BIN_OP(R, OP, DM, S, SGN) \
   R \
@@ -937,15 +937,15 @@ class boolNDArray;
   DMS_BIN_OP (R, operator +, DM, S, ) \
   DMS_BIN_OP (R, operator -, DM, S, -)
 
-#define DMS_OP_DECLS(R, DM, S) \
-  DMS_BIN_OP_DECLS(R, DM, S)
+#define DMS_OP_DECLS(R, DM, S, API) \
+  DMS_BIN_OP_DECLS(R, DM, S, API)
 
 // matrix by diagonal matrix operations.
 
-#define MDM_BIN_OP_DECLS(R, M, DM) \
-  BIN_OP_DECL (R, operator +, M, DM); \
-  BIN_OP_DECL (R, operator -, M, DM); \
-  BIN_OP_DECL (R, operator *, M, DM);
+#define MDM_BIN_OP_DECLS(R, M, DM, API) \
+  BIN_OP_DECL (R, operator +, M, DM, API); \
+  BIN_OP_DECL (R, operator -, M, DM, API); \
+  BIN_OP_DECL (R, operator *, M, DM, API);
 
 #define MDM_BIN_OP(R, OP, M, DM, OPEQ) \
 R \
@@ -1025,15 +1025,15 @@ operator * (const M& m, const DM& dm) \
   MDM_BIN_OP (R, operator -, M, DM, -=) \
   MDM_MULTIPLY_OP (R, M, DM, R_ZERO)
 
-#define MDM_OP_DECLS(R, M, DM) \
-  MDM_BIN_OP_DECLS(R, M, DM)
+#define MDM_OP_DECLS(R, M, DM, API) \
+  MDM_BIN_OP_DECLS(R, M, DM, API)
 
 // diagonal matrix by matrix operations.
 
-#define DMM_BIN_OP_DECLS(R, DM, M) \
-  BIN_OP_DECL (R, operator +, DM, M); \
-  BIN_OP_DECL (R, operator -, DM, M); \
-  BIN_OP_DECL (R, operator *, DM, M);
+#define DMM_BIN_OP_DECLS(R, DM, M, API) \
+  BIN_OP_DECL (R, operator +, DM, M, API); \
+  BIN_OP_DECL (R, operator -, DM, M, API); \
+  BIN_OP_DECL (R, operator *, DM, M, API);
 
 #define DMM_BIN_OP(R, OP, DM, M, OPEQ, PREOP) \
 R \
@@ -1113,15 +1113,15 @@ operator * (const DM& dm, const M& m) \
   DMM_BIN_OP (R, operator -, DM, M, +=, -) \
   DMM_MULTIPLY_OP (R, DM, M, R_ZERO)
 
-#define DMM_OP_DECLS(R, DM, M) \
-  DMM_BIN_OP_DECLS(R, DM, M)
+#define DMM_OP_DECLS(R, DM, M, API) \
+  DMM_BIN_OP_DECLS(R, DM, M, API)
 
 // diagonal matrix by diagonal matrix operations.
 
-#define DMDM_BIN_OP_DECLS(R, DM1, DM2) \
-  BIN_OP_DECL (R, operator +, DM1, DM2); \
-  BIN_OP_DECL (R, operator -, DM1, DM2); \
-  BIN_OP_DECL (R, product, DM1, DM2);
+#define DMDM_BIN_OP_DECLS(R, DM1, DM2, API) \
+  BIN_OP_DECL (R, operator +, DM1, DM2, API); \
+  BIN_OP_DECL (R, operator -, DM1, DM2, API); \
+  BIN_OP_DECL (R, product, DM1, DM2, API);
 
 #define DMDM_BIN_OP(R, OP, DM1, DM2, F) \
   R \
@@ -1154,8 +1154,8 @@ operator * (const DM& dm, const M& m) \
   DMDM_BIN_OP (R, operator -, DM1, DM2, mx_inline_subtract) \
   DMDM_BIN_OP (R, product,    DM1, DM2, mx_inline_multiply)
 
-#define DMDM_OP_DECLS(R, DM1, DM2) \
-  DMDM_BIN_OP_DECLS (R, DM1, DM2)
+#define DMDM_OP_DECLS(R, DM1, DM2, API) \
+  DMDM_BIN_OP_DECLS (R, DM1, DM2, API)
 
 #endif
 
