@@ -1,11 +1,11 @@
-## Copyright (C) 2005 John W. Eaton
+## Copyright (C) 2005, 2006, 2007 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
 ## Octave is free software; you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 2, or (at your option)
-## any later version.
+## the Free Software Foundation; either version 3 of the License, or (at
+## your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,25 +13,22 @@
 ## General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
-## along with Octave; see the file COPYING.  If not, write to the Free
-## Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-## 02110-1301, USA.
+## along with Octave; see the file COPYING.  If not, see
+## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {} replot ()
-## Refressh the plot window.
+## Refresh the plot window.
 ## @end deftypefn
 
 ## Author: jwe
 
 function replot ()
-
+  
   if (nargin == 0)
-    ## No semicolon following the __gnuplot_replot__ line unless you
-    ## also fix gplot.l to allow it.
-    __gnuplot_replot__
+    drawnow ();
   else
-    usage ("replot ()");
+    print_usage ();
   endif
 
 endfunction
