@@ -1,11 +1,12 @@
-## Copyright (C) 1996, 1997 John W. Eaton
+## Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2004,
+##               2005, 2006, 2007 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
 ## Octave is free software; you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 2, or (at your option)
-## any later version.
+## the Free Software Foundation; either version 3 of the License, or (at
+## your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,9 +14,8 @@
 ## General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
-## along with Octave; see the file COPYING.  If not, write to the Free
-## Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-## 02110-1301, USA.
+## along with Octave; see the file COPYING.  If not, see
+## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {} toeplitz (@var{c}, @var{r})
@@ -51,9 +51,8 @@
 ## @end group
 ## @end example
 ## @end ifinfo
+## @seealso{hankel, vander, sylvester_matrix, hilb, invhilb}
 ## @end deftypefn
-##
-## @seealso{hankel, vander, sylvester_matrix, hilb, and invhib}
 
 ## Author: jwe
 
@@ -62,7 +61,7 @@ function retval = toeplitz (c, r)
   if (nargin == 1)
     r = c;
   elseif (nargin != 2)
-    usage ("toeplitz (c, r)");
+    print_usage ();
   endif
 
   [c_nr, c_nc] = size (c);
@@ -98,7 +97,7 @@ function retval = toeplitz (c, r)
   nc = length (r);
   nr = length (c);
 
-  retval = zeros (nr, nc);
+  retval = resize (resize (c, 0), nr, nc);
 
   for i = 1:min (nc, nr)
     retval (i:nr, i) = c (1:nr-i+1);
