@@ -1,14 +1,14 @@
 /*
 
-Copyright (C) 2004 David Bateman
+Copyright (C) 1997, 1999, 2002, 2004, 2005, 2006, 2007 David Bateman
 Copyright (C) 1996, 1997 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -16,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -51,7 +50,7 @@ do_fft2 (const octave_value_list &args, const char *fcn, int type)
 
   if (nargin < 1 || nargin > 3)
     {
-      print_usage (fcn);
+      print_usage ();
       return retval;
     }
 
@@ -148,9 +147,9 @@ larger than the size of @var{a}, @var{a} is resized and padded with\n\
 zeros.\n\
 \n\
 If @var{a} is a multi-dimensional matrix, each two-dimensional sub-matrix\n\
-of @var{a} is treated seperately\n\
-@end deftypefn\n\
-@seealso {ifft2, fft, fftn fftw_wisdom}")
+of @var{a} is treated separately\n\
+@seealso {ifft2, fft, fftn, fftw}\n\
+@end deftypefn")
 {
   return do_fft2 (args, "fft2", 0);
 }
@@ -167,9 +166,9 @@ larger than the size of @var{a}, @var{a} is resized and padded with\n\
 zeros.\n\
 \n\
 If @var{a} is a multi-dimensional matrix, each two-dimensional sub-matrix\n\
-of @var{a} is treated seperately\n\
-@end deftypefn\n\
-@seealso {fft2, ifft, ifftn, fftw_wisdom}")
+of @var{a} is treated separately\n\
+@seealso {fft2, ifft, ifftn, fftw}\n\
+@end deftypefn")
 {
   return do_fft2 (args, "ifft2", 1);
 }
