@@ -461,7 +461,7 @@ gh_manager::get_handle (const std::string& go_name)
 void
 gh_manager::do_free (const graphics_handle& h)
 {
-  if (h)
+  if (h.ok ())
     {
       if (h.value () != 0)
 	{
@@ -523,7 +523,7 @@ reparent (const octave_value& ov, const std::string& who,
     {
       h = gh_manager::lookup (val);
 
-      if (h)
+      if (h.ok ())
 	{
 	  graphics_object obj = gh_manager::get_object (h);
 	  
@@ -575,6 +575,12 @@ adopt (const graphics_handle& p, const graphics_handle& h)
   parent_obj.adopt (h);
 }
 
+static bool
+is_handle (const graphics_handle& h)
+{
+  return h.ok ();
+}
+
 static bool
 is_handle (double val)
 {
@@ -586,7 +592,7 @@ is_handle (double val)
 static bool
 is_handle (const octave_value& val)
 {
-  return val.is_real_type () && is_handle (val.double_value ());
+  return val.is_real_scalar () && is_handle (val.double_value ());
 }
 
 static bool
@@ -709,7 +715,7 @@ base_properties::set_parent (const octave_value& val)
     {
       new_parent = gh_manager::lookup (tmp);
 
-      if (new_parent)
+      if (new_parent.ok ())
 	{
 	  graphics_object parent_obj = gh_manager::get_object (parent);
 
@@ -756,7 +762,7 @@ root_figure::properties::set_currentfigure (const graphics_handle& val)
   if (error_state)
     return;
 
-  if (is_handle (val))
+  if (xisnan (val.value ()) || is_handle (val))
     {
       currentfigure = val;
 
@@ -836,7 +842,7 @@ figure::properties::set_currentaxes (const graphics_handle& val)
   if (error_state)
     return;
 
-  if (is_handle (val))
+  if (xisnan (val.value ()) || is_handle (val))
     currentaxes = val;
   else
     gripe_set_invalid ("currentaxes");
@@ -1233,7 +1239,7 @@ axes::properties::set (const property_name& name, const octave_value& val)
   else if (name.compare ("xtick"))
     set_xtick (val);
   else if (name.compare ("ytick"))
-    set_xtick (val);
+    set_ytick (val);
   else if (name.compare ("ztick"))
     set_ztick (val);
   else if (name.compare ("xtickmode"))
@@ -1374,7 +1380,7 @@ axes::properties::set_defaults (base_graphics_object& obj,
 graphics_handle
 axes::properties::get_title (void) const
 {
-  if (! title)
+  if (! title.ok ())
     title = gh_manager::make_graphics_handle ("text", __myhandle__);
 
   return title;
@@ -1383,7 +1389,7 @@ axes::properties::get_title (void) const
 graphics_handle
 axes::properties::get_xlabel (void) const
 {
-  if (! xlabel)
+  if (! xlabel.ok ())
     xlabel = gh_manager::make_graphics_handle ("text", __myhandle__);
 
   return xlabel;
@@ -1392,7 +1398,7 @@ axes::properties::get_xlabel (void) const
 graphics_handle
 axes::properties::get_ylabel (void) const
 {
-  if (! ylabel)
+  if (! ylabel.ok ())
     ylabel = gh_manager::make_graphics_handle ("text", __myhandle__);
 
   return ylabel;
@@ -1401,7 +1407,7 @@ axes::properties::get_ylabel (void) const
 graphics_handle
 axes::properties::get_zlabel (void) const
 {
-  if (! zlabel)
+  if (! zlabel.ok ())
     zlabel = gh_manager::make_graphics_handle ("text", __myhandle__);
 
   return zlabel;
@@ -1592,13 +1598,13 @@ axes::properties::get (const property_name& name) const
 void
 axes::properties::remove_child (const graphics_handle& h)
 {
-  if (title && h == title)
+  if (title.ok () && h == title)
     title = gh_manager::make_graphics_handle ("text", __myhandle__);
-  else if (xlabel && h == xlabel)
+  else if (xlabel.ok () && h == xlabel)
     xlabel = gh_manager::make_graphics_handle ("text", __myhandle__);
-  else if (ylabel && h == ylabel)
+  else if (ylabel.ok () && h == ylabel)
     ylabel = gh_manager::make_graphics_handle ("text", __myhandle__);
-  else if (zlabel && h == zlabel)
+  else if (zlabel.ok () && h == zlabel)
     zlabel = gh_manager::make_graphics_handle ("text", __myhandle__);
   else
     base_properties::remove_child (h);
@@ -1819,7 +1825,7 @@ line::properties::get (void) const
   m.assign ("linewidth", linewidth);
   m.assign ("marker", marker);
   m.assign ("markeredgecolor", markeredgecolor);
-  m.assign ("markerface", markerfacecolor);
+  m.assign ("markerfacecolor", markerfacecolor);
   m.assign ("markersize", markersize);
   m.assign ("keylabel", keylabel);
 
@@ -2132,7 +2138,8 @@ patch::properties::properties (const graphics_handle& mh,
     marker ("none"),
     markeredgecolor ("auto"),
     markerfacecolor ("none"),
-    markersize (1)
+    markersize (1),
+    keylabel ("")
 { }
 
 void
@@ -2180,6 +2187,8 @@ patch::properties::set (const property_name& name,
     set_markerfacecolor (val);
   else if (name.compare ("markersize"))
     set_markersize (val);
+  else if (name.compare ("keylabel"))
+    set_keylabel (val);
   else
     {
       modified = false;
@@ -2214,6 +2223,7 @@ patch::properties::get (void) const
   m.assign ("markeredgecolor", markeredgecolor);
   m.assign ("markerface", markerfacecolor);
   m.assign ("markersize", markersize);
+  m.assign ("keylabel", keylabel);
 
   return m;
 }
@@ -2246,7 +2256,7 @@ patch::properties::get (const property_name& name) const
   else if (name.compare ("facecolor"))
     retval = facecolor;
   else if (name.compare ("facealpha"))
-    retval = facecolor;
+    retval = facealpha;
   else if (name.compare ("egdecolor"))
     retval = edgecolor;
   else if (name.compare ("linestyle"))
@@ -2261,6 +2271,8 @@ patch::properties::get (const property_name& name) const
     retval = markerfacecolor;
   else if (name.compare ("markersize"))
     retval = markersize;
+  else if (name.compare ("keylabel"))
+    retval = keylabel;
   else
     warning ("get: invalid property `%s'", name.c_str ());
 
@@ -2278,16 +2290,16 @@ patch::properties::factory_defaults (void)
   m["zdata"] = Matrix ();
   m["faces"] = Matrix ();
   m["vertices"] = Matrix ();
-  m["facecolor"] = color_property();
+  m["facecolor"] = color_property ();
   m["facealpha"] = 1.0;
-  m["edgecolor"] = color_property("black");
+  m["edgecolor"] = color_property ("black");
   m["linestyle"] = "-";
   m["linewidth"] = 0.5;
   m["marker"] = "none";
   m["markeredgecolor"] = "auto";
   m["markerfacecolor"] = "none";
   m["markersize"] = 1;
-
+  m["keylabel"] = "";
 
   return m;
 }
@@ -2302,6 +2314,16 @@ surface::properties::properties (const graphics_handle& mh,
     xdata (Matrix ()),
     ydata (Matrix ()),
     zdata (Matrix ()),
+    cdata (Matrix ()),
+    facecolor (radio_values ("{flat}|none|interp")),
+    facealpha (1.0),
+    edgecolor (color_values(0, 0, 0), radio_values ("flat|none|interp")),
+    linestyle ("-"),
+    linewidth (0.5),
+    marker ("none"),
+    markeredgecolor ("auto"),
+    markerfacecolor ("none"),
+    markersize (1),
     keylabel ("")
 { }
 
@@ -2326,6 +2348,26 @@ surface::properties::set (const property_name& name,
     set_ydata (val);
   else if (name.compare ("zdata"))
     set_zdata (val);
+  else if (name.compare ("cdata"))
+    set_cdata (val);
+  else if (name.compare ("facecolor"))
+    set_facecolor (val);
+  else if (name.compare ("facealpha"))
+    set_facealpha (val);
+  else if (name.compare ("edgecolor"))
+    set_edgecolor (val);
+  else if (name.compare ("linestyle"))
+    set_linestyle (val);
+  else if (name.compare ("linewidth"))
+    set_linewidth (val);
+  else if (name.compare ("marker"))
+    set_marker (val);
+  else if (name.compare ("markeredgecolor"))
+    set_markeredgecolor (val);
+  else if (name.compare ("markerfacecolor"))
+    set_markerfacecolor (val);
+  else if (name.compare ("markersize"))
+    set_markersize (val);
   else if (name.compare ("keylabel"))
     set_keylabel (val);
   else
@@ -2350,6 +2392,16 @@ surface::properties::get (void) const
   m.assign ("xdata", xdata);
   m.assign ("ydata", ydata);
   m.assign ("zdata", zdata);
+  m.assign ("cdata", cdata);
+  m.assign ("facecolor", facecolor);
+  m.assign ("facealpha", facealpha);
+  m.assign ("edgecolor", edgecolor);
+  m.assign ("linestyle", linestyle);
+  m.assign ("linewidth", linewidth);
+  m.assign ("marker", marker);
+  m.assign ("markeredgecolor", markeredgecolor);
+  m.assign ("markerface", markerfacecolor);
+  m.assign ("markersize", markersize);
   m.assign ("keylabel", keylabel);
 
   return m;
@@ -2374,6 +2426,26 @@ surface::properties::get (const property_name& name) const
     retval = ydata;
   else if (name.compare ("zdata"))
     retval = zdata;
+  else if (name.compare ("cdata"))
+    retval = cdata;
+  else if (name.compare ("facecolor"))
+    retval = facecolor;
+  else if (name.compare ("facealpha"))
+    retval = facealpha;
+  else if (name.compare ("edgecolor"))
+    retval = edgecolor;
+  else if (name.compare ("linestyle"))
+    retval = linestyle;
+  else if (name.compare ("linewidth"))
+    retval = linewidth;
+  else if (name.compare ("marker"))
+    retval = marker;
+  else if (name.compare ("markeredgecolor"))
+    retval = markeredgecolor;
+  else if (name.compare ("markerfacecolor"))
+    retval = markerfacecolor;
+  else if (name.compare ("markersize"))
+    retval = markersize;
   else if (name.compare ("keylabel"))
     retval = keylabel;
   else
@@ -2390,6 +2462,16 @@ surface::properties::factory_defaults (void)
   m["xdata"] = Matrix ();
   m["ydata"] = Matrix ();
   m["zdata"] = Matrix ();
+  m["cdata"] = Matrix ();
+  m["facecolor"] = color_property ();
+  m["facealpha"] = 1.0;
+  m["edgecolor"] = color_property ("black");
+  m["linestyle"] = "-";
+  m["linewidth"] = 0.5;
+  m["marker"] = "none";
+  m["markeredgecolor"] = "auto";
+  m["markerfacecolor"] = "none";
+  m["markersize"] = 1;
   m["keylabel"] = "";
 
   return m;
@@ -2651,7 +2733,7 @@ make_graphics_object (const std::string& go_name,
     {
       graphics_handle parent = gh_manager::lookup (val);
 
-      if (parent)
+      if (parent.ok ())
 	{
 	  graphics_handle h
 	    = gh_manager::make_graphics_handle (go_name, parent);
@@ -2710,7 +2792,7 @@ Undocumented internal function.\n\
 	      else
 		error ("__go_figure__: invalid figure number");
 
-	      if (! error_state && h)
+	      if (! error_state && h.ok ())
 		{
 		  adopt (0, h);
 
@@ -2813,7 +2895,7 @@ Undocumented internal function.\n\
 	{
 	  h = gh_manager::lookup (val);
 
-	  if (h)
+	  if (h.ok ())
 	    {
 	      graphics_object obj = gh_manager::get_object (h);
 
@@ -2867,7 +2949,7 @@ Undocumented internal function.\n\
 	{
 	  h = gh_manager::lookup (val);
 
-	  if (h)
+	  if (h.ok ())
 	    {
 	      graphics_object obj = gh_manager::get_object (h);
 
