@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
+              2003, 2004, 2005, 2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -594,7 +594,8 @@ find_file_to_load (const std::string& name, const std::string& orig_name)
 {
   std::string fname = name;
 
-  if (! octave_env::absolute_pathname (fname))
+  if (! (octave_env::absolute_pathname (fname)
+	 || octave_env::rooted_relative_pathname (fname)))
     {
       file_stat fs (fname);
 
@@ -612,8 +613,14 @@ find_file_to_load (const std::string& name, const std::string& orig_name)
 	}
     }
 
-  if (fname.rfind (".") == NPOS)
+  size_t dot_pos = fname.rfind (".");
+  size_t sep_pos = fname.find_last_of (file_ops::dir_sep_chars);
+    
+  if (dot_pos == NPOS || (sep_pos != NPOS && dot_pos < sep_pos))
     {
+      // Either no '.' in name or no '.' appears after last directory
+      // separator.
+
       file_stat fs (fname);
 
       if (! (fs.exists () && fs.is_reg ()))
@@ -1025,13 +1032,12 @@ glob_pattern_p (const std::string& pattern)
 static void
 do_save (std::ostream& os, const octave_value& tc,
 	 const std::string& name, const std::string& help,
-	 int global, load_save_format fmt, bool save_as_floats,
-	 bool& infnan_warned)
+	 int global, load_save_format fmt, bool save_as_floats)
 {
   switch (fmt)
     {
     case LS_ASCII:
-      save_ascii_data (os, tc, name, infnan_warned, global, 0);
+      save_ascii_data (os, tc, name, global, 0);
       break;
 
     case LS_BINARY:
@@ -1072,7 +1078,7 @@ do_save (std::ostream& os, const octave_value& tc,
 
 void
 do_save (std::ostream& os, symbol_record *sr, load_save_format fmt,
-	 bool save_as_floats, bool& infnan_warned)
+	 bool save_as_floats)
 {
   if (! sr->is_variable ())
     {
@@ -1089,8 +1095,7 @@ do_save (std::ostream& os, symbol_record *sr, load_save_format fmt,
 
       int global = sr->is_linked_to_global ();
 
-      do_save (os, tc, name, help, global, fmt, save_as_floats,
-	       infnan_warned);
+      do_save (os, tc, name, help, global, fmt, save_as_floats);
     }
 }
 
@@ -1106,11 +1111,9 @@ save_vars (std::ostream& os, const std::string& pattern,
 
   int saved = vars.length ();
 
-  bool infnan_warned = false;
-
   for (int i = 0; i < saved; i++)
     {
-      do_save (os, vars(i), fmt, save_as_floats, infnan_warned);
+      do_save (os, vars(i), fmt, save_as_floats);
 
       if (error_state)
 	break;
@@ -1240,7 +1243,7 @@ write_header (std::ostream& os, load_save_format format)
     case LS_MAT7_BINARY:
       {
 	char const * versionmagic;
-	int16_t number = *(int16_t *)"\x00\x01";
+	int16_t number = *(reinterpret_cast<const int16_t *>("\x00\x01"));
 	struct tm bdt;
 	time_t now;
 	char headertext[128];
@@ -1334,8 +1337,6 @@ dump_octave_core (std::ostream& os, const char *fname, load_save_format fmt,
 
   int num_to_save = vars.length ();
 
-  bool infnan_warned = false;
-
   double save_mem_size = 0;
 
   for (int i = 0; i < num_to_save; i++)
@@ -1363,8 +1364,7 @@ dump_octave_core (std::ostream& os, const char *fname, load_save_format fmt,
 
 		  int global = sr->is_linked_to_global ();
 
-		  do_save (os, tc, name, help, global, fmt, save_as_floats,
-			   infnan_warned);
+		  do_save (os, tc, name, help, global, fmt, save_as_floats);
 
 		  if (error_state)
 		    break;
@@ -1418,7 +1418,7 @@ dump_octave_core (void)
 #ifdef HAVE_HDF5
       if (format == LS_HDF5)
 	{
-	  hdf5_ofstream file (fname);
+	  hdf5_ofstream file (fname, mode);
 
 	  if (file.file_id >= 0)
 	    {
@@ -1648,6 +1648,9 @@ the file @file{data} in Octave's binary format.\n\
 	// don't insert any commands here!  the brace below must go
 	// with the "else" above!
 	{
+	  if (append)
+	    warning ("save: ignoring -append option for output to stdout");
+
 	  // FIXME -- should things intended for the screen end up
 	  // in a octave_value (string)?
 	  
@@ -1669,12 +1672,13 @@ the file @file{data} in Octave's binary format.\n\
 
       i++;
 
-      std::ios::openmode mode = std::ios::out;
-
       // Matlab v7 files are always compressed
       if (format == LS_MAT7_BINARY)
 	use_zlib = false;
 
+      std::ios::openmode mode
+	= append ? (std::ios::app | std::ios::ate) : std::ios::out;
+
       if (format == LS_BINARY
 #ifdef HAVE_HDF5
 	  || format == LS_HDF5
@@ -1684,17 +1688,25 @@ the file @file{data} in Octave's binary format.\n\
 	  || format == LS_MAT7_BINARY)
 	mode |= std::ios::binary;
 
-      mode |= append ? std::ios::ate : std::ios::trunc;
-
 #ifdef HAVE_HDF5
       if (format == LS_HDF5)
 	{
-	  hdf5_ofstream hdf5_file (fname.c_str ());
+	  // FIXME. It should be possible to append to HDF5 files.
+	  if (append)
+	    {
+	      error ("save: appending to HDF5 files is not implemented");
+	      return retval;
+	    }
 
-	  if (hdf5_file.file_id >= 0)
+	  bool write_header_info = ! (append && 
+				      H5Fis_hdf5 (fname.c_str ()) > 0);
+
+	  hdf5_ofstream hdf5_file (fname.c_str (), mode);
+
+	  if (hdf5_file.file_id != -1)
 	    {
 	      save_vars (argv, i, argc, hdf5_file, format,
-			 save_as_floats, true);
+			 save_as_floats, write_header_info);
 
 	      hdf5_file.close ();
 	  }
@@ -1717,7 +1729,7 @@ the file @file{data} in Octave's binary format.\n\
 	      if (file)
 		{
 		  bool write_header_info = ! file.tellp ();
-	      
+
 		  save_vars (argv, i, argc, file, format,
 			     save_as_floats, write_header_info);
 
@@ -1737,7 +1749,7 @@ the file @file{data} in Octave's binary format.\n\
 	      if (file)
 		{
 		  bool write_header_info = ! file.tellp ();
-	      
+
 		  save_vars (argv, i, argc, file, format,
 			     save_as_floats, write_header_info);
 
@@ -1790,7 +1802,7 @@ Query or set the internal variable that specifies the maximum amount\n\
 of memory (in kilobytes) of the top-level workspace that Octave will\n\
 attempt to save when writing data to the crash dump file (the name of\n\
 the file is specified by @var{octave_core_file_name}).  If\n\
-@var{octave_core_file_options} flags specifies a binary format,\n\
+@var{octave_core_file_options} flags specify a binary format,\n\
 then @var{octave_core_file_limit} will be approximately the maximum\n\
 size of the file.  If a text file format is used, then the file could\n\
 be much larger than the limit.  The default value is -1 (unlimited)\n\
@@ -1841,9 +1853,9 @@ passed to @code{strftime} and should begin with the character\n\
 the header comment is omitted from text-format data files.  The\n\
 default value is\n\
 \n\
-@example\n\
+@smallexample\n\
 \"# Created by Octave VERSION, %a %b %d %H:%M:%S %Y %Z <USER@@HOST>\"\n\
-@end example\n\
+@end smallexample\n\
 @seealso{strftime}\n\
 @end deftypefn")
 {
