@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
+              2005, 2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -30,15 +30,17 @@ class octave_value;
 class octave_value_list;
 
 #include <iostream>
+#include <sstream>
 #include <string>
+#include <map>
 
 #include "Array.h"
 #include "data-conv.h"
-#include "lo-sstream.h"
 #include "lo-utils.h"
 #include "mach-info.h"
 
 class
+OCTINTERP_API
 scanf_format_elt
 {
 public:
@@ -97,6 +99,7 @@ public:
 };
 
 class
+OCTINTERP_API
 scanf_format_list
 {
 public:
@@ -160,7 +163,7 @@ private:
   Array<scanf_format_elt*> list;
 
   // Temporary buffer.
-  OSSTREAM *buf;
+  std::ostringstream *buf;
 
   void add_elt_to_list (int width, bool discard, char type, char modifier,
 			int& num_elts,
@@ -237,6 +240,7 @@ public:
 };
 
 class
+OCTINTERP_API
 printf_format_list
 {
 public:
@@ -292,7 +296,7 @@ private:
   Array<printf_format_elt*> list;
 
   // Temporary buffer.
-  OSSTREAM *buf;
+  std::ostringstream *buf;
 
   void add_elt_to_list (int args, const std::string& flags, int fw,
 			int prec, char type, char modifier,
@@ -317,6 +321,7 @@ private:
 // Provide an interface for Octave streams.
 
 class
+OCTINTERP_API
 octave_base_stream
 {
 friend class octave_stream;
@@ -463,8 +468,6 @@ private:
   // We can always do this in terms of seek(), so the derived class
   // only has to provide that.
 
-  int rewind (void);
-
   void invalid_operation (const std::string& who, const char *rw);
 
   // No copying!
@@ -475,6 +478,7 @@ private:
 };
 
 class
+OCTINTERP_API
 octave_stream
 {
 public:
@@ -580,9 +584,15 @@ public:
 
   static std::string mode_as_string (int mode);
 
-  std::istream *input_stream (void) { return rep ? rep->input_stream () : 0; }
+  std::istream *input_stream (void)
+  {
+    return rep ? rep->input_stream () : 0;
+  }
 
-  std::ostream *output_stream (void) { return rep ? rep->output_stream () : 0; }
+  std::ostream *output_stream (void)
+  {
+    return rep ? rep->output_stream () : 0;
+  }
 
   void clearerr (void) { if (rep) rep->clearerr (); }
 
@@ -591,9 +601,7 @@ private:
   // The actual representation of this stream.
   octave_base_stream *rep;
 
-  void invalid_stream_error (const std::string& who) const;
-
-  bool stream_ok (const std::string& who, bool clear = true) const
+  bool stream_ok (bool clear = true) const
     {
       bool retval = true;
 
@@ -603,10 +611,7 @@ private:
 	    rep->clear ();
 	}
       else
-	{
-	  retval = false;
-	  invalid_stream_error (who);
-	}
+	retval = false;
 
       return retval;
     }
@@ -619,11 +624,12 @@ private:
 };
 
 class
+OCTINTERP_API
 octave_stream_list
 {
 protected:
 
-  octave_stream_list (void) : list (32), curr_len (0) { }
+  octave_stream_list (void) : list () { }
 
 public:
 
@@ -631,7 +637,7 @@ public:
 
   static bool instance_ok (void);
 
-  static int insert (const octave_stream& os);
+  static int insert (octave_stream& os);
 
   static octave_stream
   lookup (int fid, const std::string& who = std::string ());
@@ -656,13 +662,13 @@ public:
 
 private:
 
-  Array<octave_stream> list;
+  typedef std::map<int, octave_stream> ostrl_map;
 
-  int curr_len;
+  ostrl_map list;
 
   static octave_stream_list *instance;
 
-  int do_insert (const octave_stream& os);
+  int do_insert (octave_stream& os);
 
   octave_stream do_lookup (int fid, const std::string& who = std::string ()) const;
   octave_stream do_lookup (const octave_value& fid,
