@@ -1,13 +1,13 @@
 /*
 
-Copyright (C) 2003 John W. Eaton
+Copyright (C) 2003, 2004, 2005, 2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +15,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -41,21 +40,22 @@ octave_fcn_handle : public octave_base_value
 {
 public:
   octave_fcn_handle (void)
-    : fcn (), nm () { }
+    : warn_reload (true), fcn (), nm () { }
 
   octave_fcn_handle (const std::string& n)
-    : fcn (), nm (n) { }
+    : warn_reload (true), fcn (), nm (n) { }
 
   octave_fcn_handle (const octave_value& f,  const std::string& n)
-    : fcn (f), nm (n) { }
+    : warn_reload (true), fcn (f), nm (n) { }
 
   octave_fcn_handle (const octave_fcn_handle& fh)
-    : octave_base_value (fh), fcn (fh.fcn), nm (fh.nm) { }
+    : octave_base_value (fh), warn_reload (fh.warn_reload),
+      fcn (fh.fcn), nm (fh.nm) { }
 
   ~octave_fcn_handle (void) { }
 
-  octave_value *clone (void) const { return new octave_fcn_handle (*this); }
-  octave_value *empty_clone (void) const { return new octave_fcn_handle (); }
+  octave_base_value *clone (void) const { return new octave_fcn_handle (*this); }
+  octave_base_value *empty_clone (void) const { return new octave_fcn_handle (); }
 
   octave_value subsref (const std::string&,
 			const std::list<octave_value_list>&)
@@ -72,17 +72,21 @@ public:
 
   bool is_function_handle (void) const { return true; }
 
+  dim_vector dims (void) const { static dim_vector dv (1, 1); return dv; }
+
   octave_function *function_value (bool = false)
     { return fcn.function_value (); }
 
+  octave_user_function *user_function_value (bool = false)
+    { return fcn.user_function_value (); }
+
   octave_fcn_handle *fcn_handle_value (bool = false) { return this; }
 
   octave_value fcn_val (void) const { return fcn; }
 
   std::string fcn_name (void) const { return nm; }
 
-  bool save_ascii (std::ostream& os, bool& infnan_warned,
-		 bool strip_nan_and_inf);
+  bool save_ascii (std::ostream& os);
 
   bool load_ascii (std::istream& is);
 
@@ -103,10 +107,19 @@ public:
 
 private:
 
+  bool set_fcn (const std::string &octaveroot, const std::string& fpath);
+
   DECLARE_OCTAVE_ALLOCATOR
 
   DECLARE_OV_TYPEID_FUNCTIONS_AND_DATA
 
+  // If TRUE, print a warning if the pointed-to fucntion is out of
+  // date.  This variable may be removed when updating is properly
+  // implemented.
+  mutable bool warn_reload;
+
+  void reload_warning (const std::string& fcn_type) const;
+
 protected:
 
   // The function we are handling.
