@@ -367,9 +367,12 @@ octave_struct::subsasgn (const std::string& type,
 	      {
 		Cell tmp_cell = Cell (t_rhs.list_value ());
 
-		// FIXME -- shouldn't care if the dimensions of the
-		// RHS don't match the dimensions of the subscriped
-		// LHS.
+		// The shape of the RHS is irrelevant, we just want
+		// the number of elements to agree and to preserve the
+		// shape of the left hand side of the assignment.
+
+		if (numel () == tmp_cell.numel ())
+		  tmp_cell = tmp_cell.reshape (dims ());
 
 		map.assign (key, tmp_cell);
 	      }
@@ -400,6 +403,68 @@ octave_struct::subsasgn (const std::string& type,
   return retval;
 }
 
+octave_value
+octave_struct::do_index_op (const octave_value_list& idx, bool resize_ok)
+{
+  octave_value retval;
+
+  octave_idx_type n_idx = idx.length ();
+
+  int nd = map.ndims ();
+
+  switch (n_idx)
+    {
+    case 0:
+      retval = map;
+      break;
+
+    case 1:
+      {
+	idx_vector i = idx (0).index_vector ();
+
+	if (! error_state)
+	  retval = map.index (i, resize_ok, Cell::resize_fill_value ());
+      }
+      break;
+
+    default:
+      {
+	if (n_idx == 2 && nd == 2)
+	  {
+	    idx_vector i = idx (0).index_vector ();
+
+	    if (! error_state)
+	      {
+		idx_vector j = idx (1).index_vector ();
+
+		if (! error_state)
+		  retval = map.index (i, j, resize_ok,
+				      Cell::resize_fill_value ());
+	      }
+	  }
+	else
+	  {
+	    Array<idx_vector> idx_vec (n_idx);
+
+	    for (octave_idx_type i = 0; i < n_idx; i++)
+	      {
+		idx_vec(i) = idx(i).index_vector ();
+
+		if (error_state)
+		  break;
+	      }
+
+	    if (! error_state)
+	      retval = map.index (idx_vec, resize_ok,
+				  Cell::resize_fill_value ());
+	  }
+      }
+      break;
+    }
+
+  return retval;
+}
+
 size_t
 octave_struct::byte_size (void) const
 {
@@ -829,7 +894,8 @@ that is @code{numel (@var{fields}) == size (@var{cell}, @var{dim})}.\n\
 \n\
 @example\n\
 @group\n\
-A = cell2struct (@{'Peter', 'Hannah', 'Robert'; 185, 170, 168@},\n\
+A = cell2struct (@{'Peter', 'Hannah', 'Robert';\n\
+                   185, 170, 168@},\n\
                  @{'Name','Height'@}, 1);\n\
 A(1)\n\
 @result{} ans =\n\
