@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1996, 1997, 1998, 1999, 2000, 2002, 2003, 2004, 2005,
+              2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -49,7 +49,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "variables.h"
 
 // Maximum nesting level for functions called recursively.
-static int Vmax_recursion_depth;
+static int Vmax_recursion_depth = 256;
 
 // User defined functions.
 
@@ -59,6 +59,12 @@ DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA (octave_user_function,
 				     "user-defined function",
 				     "user-defined function");
 
+DEFINE_OCTAVE_ALLOCATOR (octave_user_script);
+
+DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA (octave_user_script,
+				     "user-defined script",
+				     "user-defined script");
+
 // Ugh.  This really needs to be simplified (code/data?
 // extrinsic/intrinsic state?).
 
@@ -67,29 +73,24 @@ octave_user_function::octave_user_function
    tree_statement_list *cl, symbol_table *st)
   : octave_function (std::string (), std::string ()),
     param_list (pl), ret_list (rl), cmd_list (cl),
-    sym_tab (st), lead_comm (), trail_comm (), file_name (),
-    t_parsed (static_cast<time_t> (0)),
+    local_sym_tab (st), lead_comm (), trail_comm (), file_name (),
+    parent_name (), t_parsed (static_cast<time_t> (0)),
     t_checked (static_cast<time_t> (0)),
-    system_fcn_file (false), call_depth (0),
-    num_named_args (0), nested_function (false),
-    args_passed (), num_args_passed (0),
-    curr_va_arg_number (0), vr_list (0), symtab_entry (0),
-    argn_sr (0), nargin_sr (0), nargout_sr (0), varargin_sr (0)
+    system_fcn_file (false), call_depth (0), num_named_args (0),
+    nested_function (false), inline_function (false), args_passed (),
+    num_args_passed (0), symtab_entry (0), argn_sr (0),
+    nargin_sr (0), nargout_sr (0), varargin_sr (0)
 {
   if (param_list)
-    {
-      num_named_args = param_list->length ();
-      curr_va_arg_number = num_named_args;
-    }
+    num_named_args = param_list->length ();
 }
 
 octave_user_function::~octave_user_function (void)
 {
   delete param_list;
   delete ret_list;
-  delete sym_tab;
+  delete local_sym_tab;
   delete cmd_list;
-  delete vr_list;
   delete lead_comm;
   delete trail_comm;
 }
@@ -99,9 +100,6 @@ octave_user_function::define_ret_list (tree_parameter_list *t)
 {
   ret_list = t;
 
-  if (ret_list && ret_list->takes_varargs ())
-    vr_list = new tree_va_return_list;
- 
   return this;
 }
 
@@ -118,7 +116,7 @@ octave_user_function::mark_as_system_fcn_file (void)
     {
       // We really should stash the whole path to the file we found,
       // when we looked it up, to avoid possible race conditions...
-      // XXX FIXME XXX
+      // FIXME
       //
       // We probably also don't need to get the library directory
       // every time, but since this function is only called when the
@@ -140,18 +138,10 @@ octave_user_function::takes_varargs (void) const
   return (param_list && param_list->takes_varargs ());
 }
 
-octave_value
-octave_user_function::octave_va_arg (void)
+bool
+octave_user_function::takes_var_return (void) const
 {
-  octave_value retval;
-
-  if (curr_va_arg_number < num_args_passed)
-    retval = args_passed (curr_va_arg_number++);
-  else
-    ::error ("va_arg: error getting arg number %d -- only %d provided",
-	     curr_va_arg_number + 1, num_args_passed);
-
-  return retval;
+  return (ret_list && ret_list->takes_varargs ());
 }
 
 octave_value_list
@@ -173,69 +163,6 @@ octave_user_function::octave_all_va_args (void)
   return retval;
 }
 
-bool
-octave_user_function::takes_var_return (void) const
-{
-  return (ret_list && ret_list->takes_varargs ());
-}
-
-void
-octave_user_function::octave_vr_val (const octave_value& val)
-{
-  // Use != here to avoid possible conversion to int of smaller type
-  // than the vr_list pointer.
-
-  assert (vr_list != 0);
-
-  vr_list->append (val);
-}
-
-void
-octave_user_function::varargout_to_vr_val (void)
-{
-  assert (vr_list && vr_list->empty ());
-
-  symbol_record *sr = sym_tab->lookup ("varargout");
-
-  if (sr && sr->is_variable ())
-    {
-      octave_value v = sr->def ();
-
-      Cell c = v.cell_value ();
-
-      if (! error_state)
-	{
-	  // XXX FIXME XXX -- should varargout be required to be a
-	  // cell array with a single row or column?  If not, should
-	  // we have a cleaner way of doing this operation?
-
-	  int n = c.length ();
-
-	  const octave_value *d = c.data ();
-
-	  for (int i = 0; i < n; i++)
-	    vr_list->append (d[i]);
-	}
-      else
-	error ("expecting varargout to be a cell array object");
-    }
-}
-
-bool
-octave_user_function::has_varargout (void) const
-{
-  bool retval = false;
-
-  if (takes_var_return ())
-    {
-      symbol_record *sr = sym_tab->lookup ("varargout");
-
-      retval = (sr && sr->is_variable ());
-    }
-
-  return retval;
-}
-
 // For unwind protect.
 
 static void
@@ -245,14 +172,6 @@ pop_symbol_table_context (void *table)
   tmp->pop_context ();
 }
 
-static void
-delete_vr_list (void *list)
-{
-  tree_va_return_list *tmp = static_cast<tree_va_return_list *> (list);
-  tmp->clear ();
-  delete tmp;
-}
-
 static void
 clear_symbol_table (void *table)
 {
@@ -314,7 +233,7 @@ octave_user_function::subsref (const std::string& type,
       panic_impossible ();
     }
 
-  // XXX FIXME XXX -- perhaps there should be an
+  // FIXME -- perhaps there should be an
   // octave_value_list::next_subsref member function?  See also
   // octave_builtin::subsref.
 
@@ -358,31 +277,15 @@ octave_user_function::do_multi_index_op (int nargout,
 
   if (call_depth > 1)
     {
-      sym_tab->push_context ();
-      unwind_protect::add (pop_symbol_table_context, sym_tab);
-
-      if (vr_list)
-	{
-	  // Push new vr_list.
-
-	  unwind_protect_ptr (vr_list);
-	  vr_list = new tree_va_return_list;
-
-	  // Clear and delete the new one before restoring the old
-	  // one.
-
-	  unwind_protect::add (delete_vr_list, vr_list);
-	}
+      local_sym_tab->push_context ();
+      unwind_protect::add (pop_symbol_table_context, local_sym_tab);
     }
 
-  if (vr_list)
-    vr_list->clear ();
-
   install_automatic_vars ();
 
   // Force symbols to be undefined again when this function exits.
 
-  unwind_protect::add (clear_symbol_table, sym_tab);
+  unwind_protect::add (clear_symbol_table, local_sym_tab);
 
   // Save old and set current symbol table context, for
   // eval_undefined_error().
@@ -391,17 +294,16 @@ octave_user_function::do_multi_index_op (int nargout,
   curr_caller_sym_tab = curr_sym_tab;
 
   unwind_protect_ptr (curr_sym_tab);
-  curr_sym_tab = sym_tab;
+  curr_sym_tab = local_sym_tab;
 
-  unwind_protect_ptr (curr_function);
-  unwind_protect_ptr (curr_caller_function);
   unwind_protect_ptr (curr_caller_statement);
-
   curr_caller_statement = curr_statement;
-  curr_caller_function = curr_function;
-  curr_function = this;
 
-  if (! is_nested_function ())
+  octave_call_stack::push (this);
+
+  unwind_protect::add (octave_call_stack::unwind_pop, 0);
+
+  if (! (is_nested_function () || is_inline_function ()))
     {
       unwind_protect_ptr (curr_parent_function);
       curr_parent_function = this;
@@ -418,10 +320,6 @@ octave_user_function::do_multi_index_op (int nargout,
   unwind_protect_int (num_args_passed);
   num_args_passed = nargin;
 
-  unwind_protect_int (curr_va_arg_number);
-
-  curr_va_arg_number = num_named_args;
-
   if (param_list && ! param_list->varargs_only ())
     {
       param_list->define_from_arg_vector (args);
@@ -458,12 +356,14 @@ octave_user_function::do_multi_index_op (int nargout,
     unwind_protect_bool (evaluating_function_body);
     evaluating_function_body = true;
 
-    octave_value_list tmp = cmd_list->eval ();
+    if (is_inline_function ())
+      {
+	assert (cmd_list->length () == 1);
 
-    octave_value last_computed_value;
-
-    if (! tmp.empty ())
-      last_computed_value = tmp(0);
+	retval = cmd_list->eval (false, nargout);
+      }
+    else
+      cmd_list->eval ();
 
     if (echo_commands)
       print_code_function_trailer ();
@@ -482,14 +382,29 @@ octave_user_function::do_multi_index_op (int nargout,
     
     // Copy return values out.
 
-    if (ret_list)
+    if (ret_list && ! is_inline_function ())
       {
 	ret_list->initialize_undefined_elements (my_name, nargout, Matrix ());
 
-	if (has_varargout ())
-	  varargout_to_vr_val ();
+	Cell varargout;
 
-	retval = ret_list->convert_to_const_vector (vr_list);
+	if (ret_list->takes_varargs ())
+	  {
+	    symbol_record *sr = local_sym_tab->lookup ("varargout");
+
+	    if (sr && sr->is_variable ())
+	      {
+		octave_value v = sr->def ();
+
+		varargout = v.cell_value ();
+
+		if (error_state)
+		  error ("expecting varargout to be a cell array object");
+	      }
+	  }
+
+	if (! error_state)
+	  retval = ret_list->convert_to_const_vector (varargout);
       }
   }
 
@@ -531,8 +446,8 @@ octave_user_function::accept (tree_walker& tw)
 void
 octave_user_function::print_symtab_info (std::ostream& os) const
 {
-  if (sym_tab)
-    sym_tab->print_info (os);
+  if (local_sym_tab)
+    local_sym_tab->print_info (os);
   else
     warning ("%s: no symbol table info!", my_name.c_str ());
 }
@@ -540,7 +455,7 @@ octave_user_function::print_symtab_info (std::ostream& os) const
 void
 octave_user_function::print_code_function_header (void)
 {
-  tree_print_code tpc (octave_stdout, Vps4);
+  tree_print_code tpc (octave_stdout, VPS4);
 
   tpc.visit_octave_user_function_header (*this);
 }
@@ -548,7 +463,7 @@ octave_user_function::print_code_function_header (void)
 void
 octave_user_function::print_code_function_trailer (void)
 {
-  tree_print_code tpc (octave_stdout, Vps4);
+  tree_print_code tpc (octave_stdout, VPS4);
 
   tpc.visit_octave_user_function_trailer (*this);
 }
@@ -556,14 +471,14 @@ octave_user_function::print_code_function_trailer (void)
 void
 octave_user_function::install_automatic_vars (void)
 {
-  if (sym_tab)
+  if (local_sym_tab)
     {
-      argn_sr = sym_tab->lookup ("argn", true);
-      nargin_sr = sym_tab->lookup ("__nargin__", true);
-      nargout_sr = sym_tab->lookup ("__nargout__", true);
+      argn_sr = local_sym_tab->lookup ("argn", true);
+      nargin_sr = local_sym_tab->lookup ("__nargin__", true);
+      nargout_sr = local_sym_tab->lookup ("__nargout__", true);
 
       if (takes_varargs ())
-	varargin_sr = sym_tab->lookup ("varargin", true);
+	varargin_sr = local_sym_tab->lookup ("varargin", true);
     }
 }
 
@@ -600,8 +515,7 @@ At the top level, return the number of command line arguments passed to\n\
 Octave.  If called with the optional argument @var{fcn_name}, return the\n\
 maximum number of arguments the named function can accept, or -1 if the\n\
 function accepts a variable number of arguments.\n\
-\n\
-@seealso{nargout, varargin, and varargout}\n\
+@seealso{nargout, varargin, varargout}\n\
 @end deftypefn")
 {
   octave_value retval;
@@ -642,7 +556,7 @@ function accepts a variable number of arguments.\n\
       retval = sr ? sr->def () : 0;
     }
   else
-    print_usage ("nargin");
+    print_usage ();
 
   return retval;
 }
@@ -674,8 +588,7 @@ will cause @code{nargout} to return 2 inside the function\n\
 @code{f}.\n\
 \n\
 At the top level, @code{nargout} is undefined.\n\
-\n\
-@seealso{nargin, varargin, and varargout}\n\
+@seealso{nargin, varargin, varargout}\n\
 @end deftypefn")
 {
   octave_value retval;
@@ -721,161 +634,21 @@ At the top level, @code{nargout} is undefined.\n\
 	error ("nargout: invalid call at top level");
     }
   else
-    print_usage ("nargout");
+    print_usage ();
 
   return retval;
 }
 
-DEFUNX ("va_arg", Fva_arg, args, ,
+DEFUN (max_recursion_depth, args, nargout,
   "-*- texinfo -*-\n\
-@deftypefn {Built-in Function} {} va_arg ()\n\
-Return the value of the next available argument and move the internal\n\
-pointer to the next argument.  It is an error to call @code{va_arg}\n\
-when ther eare no more arguments available, or in a function that\n\
-has not been declared to take a variable number of parameters.\n\
+@deftypefn {Built-in Function} {@var{val} =} max_recursion_depth ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} max_recursion_depth (@var{new_val})\n\
+Query or set the internal limit on the number of times a function may\n\
+be called recursively.  If the limit is exceeded, an error message is\n\
+printed and control returns to the top level.\n\
 @end deftypefn")
 {
-  static bool warned = false;
-
-  if (! warned)
-    {
-      ::warning ("va_arg is deprecated; use varargin instead");
-      warned = true;
-    }
-
-  octave_value_list retval;
-
-  int nargin = args.length ();
-
-  if (nargin == 0)
-    {
-      if (curr_function)
-	{
-	  if (curr_function->takes_varargs ())
-	    retval = curr_function->octave_va_arg ();
-	  else
-	    {
-	      ::error ("va_arg only valid within function taking variable");
-	      ::error ("number of arguments");
-	    }
-	}
-      else
-	::error ("va_arg only valid within function body");
-    }
-  else
-    print_usage ("va_arg");
-
-  return retval;
-}
-
-DEFUN (va_start, args, ,
-  "-*- texinfo -*-\n\
-@deftypefn {Built-in Function} {} va_start ()\n\
-Position an internal pointer to the first unnamed argument in\n\
-functions that have been declared to accept a variable number of\n\
-arguments.  It is an error to call @code{va_start} in a function\n\
-that has not been declared to take a variable number of parameters.\n\
-@end deftypefn")
-{
-  static bool warned = false;
-
-  if (! warned)
-    {
-      ::warning ("va_start is deprecated; use varargin instead");
-      warned = true;
-    }
-
-  octave_value_list retval;
-
-  int nargin = args.length ();
-
-  if (nargin == 0)
-    {
-      if (curr_function)
-	{
-	  if (curr_function->takes_varargs ())
-	    curr_function->octave_va_start ();
-	  else
-	    {
-	      ::error ("va_start only valid within function taking variable");
-	      ::error ("number of arguments");
-	    }
-	}
-      else
-	::error ("va_start only valid within function body");
-    }
-  else
-    print_usage ("va_start");
-
-  return retval;
-}
-
-DEFUN (vr_val, args, ,
-  "-*- texinfo -*-\n\
-@deftypefn {Built-in Function} {} vr_val (@var{x})\n\
-Each time this function is called, it places the value of its argument\n\
-at the end of the list of values to return from the current\n\
-function.  Once @code{vr_val} has been called, there is no way to go\n\
-back to the beginning of the list and rewrite any of the return\n\
-values.  This function may only be called within functions that have\n\
-been declared to return an unspecified number of output arguments.\n\
-@end deftypefn")
-{
-  static bool warned = false;
-
-  if (! warned)
-    {
-      ::warning ("vr_val is deprecated; use varargout instead");
-      warned = true;
-    }
-
-  octave_value_list retval;
-
-  int nargin = args.length ();
-
-  if (nargin == 1)
-    {
-      if (curr_function)
-	{
-	  if (curr_function->has_varargout ())
-	    ::error ("vr_val and varargout cannot both be used in the same function");
-	  else if (curr_function->takes_var_return ())
-	    curr_function->octave_vr_val (args(0));
-	  else
-	    {
-	      ::error ("vr_val only valid within function declared to");
-	      ::error ("produce a variable number of values");
-	    }
-	}
-      else
-	::error ("vr_val only valid within function body");
-    }
-  else
-    print_usage ("vr_val");
-
-  return retval;
-}
-
-static int
-max_recursion_depth (void)
-{
-  Vmax_recursion_depth = check_preference ("max_recursion_depth");
-
-  return 0;
-}
-
-void
-symbols_of_ov_usr_fcn (void)
-{
-  DEFVAR (max_recursion_depth, 256.0, max_recursion_depth,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} max_recursion_depth\n\
-Limit the number of times a function may be called recursively.\n\
-If the limit is exceeded, an error message is printed and control\n\
-returns to the top level.\n\
-\n\
-The default value is 256.\n\
-@end defvr");
+  return SET_INTERNAL_VARIABLE (max_recursion_depth);
 }
 
 /*
