@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
+              2004, 2005, 2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -33,6 +33,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 
 #include <fstream>
 #include <iostream>
+#include <sstream>
 #include <string>
 
 #ifdef HAVE_UNISTD_H
@@ -46,7 +47,6 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "file-ops.h"
 #include "lo-error.h"
 #include "lo-mappers.h"
-#include "lo-sstream.h"
 #include "oct-env.h"
 #include "quit.h"
 #include "str-vec.h"
@@ -90,15 +90,91 @@ bool octave_initialized = false;
 // Current command to execute.
 tree_statement_list *global_command = 0;
 
-// Pointer to function that is currently being evaluated.
-octave_function *curr_function = 0;
-
-// Pointer to caller of curr_function.
-octave_function *curr_caller_function = 0;
-
 // Pointer to parent function that is currently being evaluated.
 octave_function *curr_parent_function = 0;
 
+octave_call_stack *octave_call_stack::instance = 0;
+
+octave_function *
+octave_call_stack::do_caller (void)
+{
+  octave_function *retval = 0;
+
+  if (cs.size () > 1)
+    {
+      iterator p = cs.begin ();
+      retval = *++p;
+    }
+
+  return retval;
+}
+
+octave_user_script *
+octave_call_stack::do_caller_user_script (void)
+{
+  octave_user_script *retval = 0;
+
+  for (iterator p = cs.begin (); p != cs.end (); p++)
+    {
+      octave_function *f = *p;
+
+      if (f && f->is_user_script ())
+	{
+	  retval = dynamic_cast<octave_user_script *> (f);
+	  break;
+	}
+    }
+
+  return retval;
+}
+
+octave_user_function *
+octave_call_stack::do_caller_user_function (void)
+{
+  octave_user_function *retval = 0;
+
+  for (iterator p = cs.begin (); p != cs.end (); p++)
+    {
+      octave_function *f = *p;
+
+      if (f && f->is_user_function ())
+	{
+	  retval = dynamic_cast<octave_user_function *> (f);
+	  break;
+	}
+    }
+
+  return retval;
+}
+
+octave_function *
+octave_call_stack::do_caller_user_script_or_function (void)
+{
+  octave_function *retval = 0;
+
+  for (iterator p = cs.begin (); p != cs.end (); p++)
+    {
+      octave_function *f = *p;
+
+      if (f && (f->is_user_script () || f->is_user_function ()))
+	{
+	  retval = f;
+	  break;
+	}
+    }
+
+  return retval;
+}
+
+void
+octave_call_stack::unwind_pop_script (void *)
+{
+  octave_function *f = top ();
+  pop ();
+  assert (f && f->is_user_script ());
+  delete f;
+}
+
 static void
 recover_from_exception (void)
 {
@@ -193,13 +269,14 @@ main_loop (void)
       catch (octave_interrupt_exception)
 	{
 	  recover_from_exception ();
-	  std::cout << "\n";
+	  octave_stdout << "\n";
 	}
       catch (std::bad_alloc)
 	{
 	  recover_from_exception ();
 	  std::cerr
-	    << "error: memory exhausted -- trying to return to prompt\n";
+	    << "error: memory exhausted or requested size too large for range of Octave's index type -- trying to return to prompt"
+	    << std::endl;
 	}
     }
   while (retval == 0);
@@ -214,41 +291,18 @@ clean_up_and_exit (int retval)
 {
   do_octave_atexit ();
 
+  sysdep_cleanup ();
+
   exit (retval == EOF ? 0 : retval);
 }
 
-DEFCMD (casesen, args, ,
-  "-*- texinfo -*-\n\
-@deffn {Command} casesen arg\n\
-Provided for compatibility with Matlab, but does nothing.\n\
-@end deffn")
-{
-  octave_value_list retval;
-
-  int argc = args.length () + 1;
-
-  string_vector argv = args.make_argv ("casesen");
-
-  if (error_state)
-    return retval;
-
-  if (argc == 1 || (argc > 1 && argv[1] == "off"))
-    warning ("casesen: sorry, Octave is always case sensitive");
-  else if (argc > 1 && argv[1] == "on")
-    ; // ok.
-  else
-    print_usage ("casesen");
-
-  return retval;
-}
-
 DEFUN (quit, args, nargout,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} exit (@var{status})\n\
 @deftypefnx {Built-in Function} {} quit (@var{status})\n\
 Exit the current Octave session.  If the optional integer value\n\
 @var{status} is supplied, pass that value to the operating system as the\n\
-Octave's exit status.\n\
+Octave's exit status.  The default value is zero.\n\
 @end deftypefn")
 {
   octave_value_list retval;
@@ -290,7 +344,7 @@ Describe the conditions for copying and distributing Octave.\n\
 \n\
 This program is free software; you can redistribute it and/or modify\n\
 it under the terms of the GNU General Public License as published by\n\
-the Free Software Foundation; either version 2 of the License, or\n\
+the Free Software Foundation; either version 3 of the License, or\n\
 (at your option) any later version.\n\
 \n\
 This program is distributed in the hope that it will be useful,\n\
@@ -299,9 +353,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
 GNU General Public License for more details.\n\
 \n\
 You should have received a copy of the GNU General Public License\n\
-along with this program. If not, write to the Free Software\n\
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA\n\
-02110-1301, USA.\n\
+along with this program.  If not, see <http://www.gnu.org/licenses/>.\n\
 \n";
 
   return retval;
@@ -319,6 +371,28 @@ cleanup_iprocstream (void *p)
   delete cmd;
 }
 
+static int
+wait_for_input (int fid)
+{
+  int retval = -1;
+
+#if defined (HAVE_SELECT)
+  if (fid >= 0)
+    {
+      fd_set set;
+
+      FD_ZERO (&set);
+      FD_SET (fid, &set);
+
+      retval = select (FD_SETSIZE, &set, 0, 0, 0);
+    }
+#else
+  retval = 1;
+#endif
+
+  return retval;
+}
+
 static octave_value_list
 run_command_and_return_output (const std::string& cmd_str)
 {
@@ -332,15 +406,9 @@ run_command_and_return_output (const std::string& cmd_str)
 
       if (*cmd)
 	{
-	  OSSTREAM output_buf;
+	  int fid = cmd->file_number ();
 
-	  // XXX FIXME XXX -- Perhaps we should read more than one
-	  // character at a time and find a way to avoid the call to
-	  // octave_usleep as well?
-
-	  // This is a bit of a kluge...
-
-	  octave_usleep (100);
+	  std::ostringstream output_buf;
 
 	  char ch;
 
@@ -354,7 +422,8 @@ run_command_and_return_output (const std::string& cmd_str)
 		    {
 		      cmd->clear ();
 
-		      octave_usleep (100);
+		      if (wait_for_input (fid) != 1)
+			break;			
 		    }
 		  else
 		    break;
@@ -368,12 +437,8 @@ run_command_and_return_output (const std::string& cmd_str)
 	  else
 	    cmd_status = 127;
 
-	  output_buf << OSSTREAM_ENDS;
-
-	  retval(1) = (double) cmd_status;
-	  retval(0) = OSSTREAM_STR (output_buf);
-
-	  OSSTREAM_FREEZE (output_buf);
+	  retval(0) = (double) cmd_status;
+	  retval(1) = output_buf.str ();
 	}
 
       unwind_protect::run ();
@@ -415,12 +480,12 @@ or\n\
 printf (\"%s\n\", system (cmd, 1));\n\
 @end example\n\
 \n\
-The @code{system} function can return two values.  The first is any\n\
-output from the command that was written to the standard output stream,\n\
-and the second is the output status of the command.  For example,\n\
+The @code{system} function can return two values.  The first is the\n\
+exit status of the command and the second is any output from the\n\
+command that was written to the standard output stream.  For example,\n\
 \n\
 @example\n\
-[output, status] = system (\"echo foo; exit 2\");\n\
+[status, output] = system (\"echo foo; exit 2\");\n\
 @end example\n\
 \n\
 @noindent\n\
@@ -436,7 +501,7 @@ variable @code{status} to the integer @samp{2}.\n\
 
   if (nargin > 0 && nargin < 4)
     {
-      bool return_output = (nargout > 0 || nargin > 1);
+      bool return_output = (nargout > 1 || nargin > 1);
 
       std::string cmd_str = args(0).string_value ();
 
@@ -466,8 +531,15 @@ variable @code{status} to the integer @samp{2}.\n\
 
       if (! error_state)
 	{
+#if defined (__WIN32__) && ! defined (__CYGWIN__)
+	  // Work around weird double-quote handling on Windows systems.
+          if (type == et_sync)
+            cmd_str = "\"" + cmd_str + "\"";
+#endif
+
 	  if (type == et_async)
 	    {
+	      // FIXME -- maybe this should go in sysdep.cc?
 #ifdef HAVE_FORK
 	      pid_t pid = fork ();
 
@@ -475,15 +547,32 @@ variable @code{status} to the integer @samp{2}.\n\
 		error ("system: fork failed -- can't create child process");
 	      else if (pid == 0)
 		{
-		  // XXX FIXME XXX -- should probably replace this
+		  // FIXME -- should probably replace this
 		  // call with something portable.
 
-		  execl ("/bin/sh", "sh", "-c", cmd_str.c_str (), 0);
+		  execl ("/bin/sh", "sh", "-c", cmd_str.c_str (),
+			 static_cast<void *> (0));
 
 		  panic_impossible ();
 		}
 	      else
 		retval(0) = pid;
+#elif defined (__WIN32__)
+              STARTUPINFO si;
+              PROCESS_INFORMATION pi;
+              ZeroMemory (&si, sizeof (si));
+              ZeroMemory (&pi, sizeof (pi));
+	      OCTAVE_LOCAL_BUFFER (char, xcmd_str, cmd_str.length()+1);
+	      strcpy (xcmd_str, cmd_str.c_str ());
+
+              if (! CreateProcess (0, xcmd_str, 0, 0, FALSE, 0, 0, 0, &si, &pi))
+                error ("system: CreateProcess failed -- can't create child process");
+              else
+                {
+                  retval(0) = pi.dwProcessId;
+                  CloseHandle (pi.hProcess);
+                  CloseHandle (pi.hThread);
+                }
 #else
  	      error ("asynchronous system calls are not supported");
 #endif
@@ -507,7 +596,7 @@ variable @code{status} to the integer @samp{2}.\n\
 	}
     }
   else
-    print_usage ("system");
+    print_usage ();
 
   unwind_protect::run_frame ("Fsystem");
 
@@ -516,9 +605,9 @@ variable @code{status} to the integer @samp{2}.\n\
 
 DEFALIAS (shell_cmd, system);
 
-// XXX FIXME XXX -- this should really be static, but that causes
+// FIXME -- this should really be static, but that causes
 // problems on some systems.
-std::stack<std::string> octave_atexit_functions;
+std::list<std::string> octave_atexit_functions;
 
 void
 do_octave_atexit (void)
@@ -527,9 +616,9 @@ do_octave_atexit (void)
 
   while (! octave_atexit_functions.empty ())
     {
-      std::string fcn = octave_atexit_functions.top ();
+      std::string fcn = octave_atexit_functions.front ();
 
-      octave_atexit_functions.pop ();
+      octave_atexit_functions.pop_front ();
 
       reset_error_handler ();
 
@@ -542,9 +631,14 @@ do_octave_atexit (void)
     {
       deja_vu = true;
 
+      // Do this explicitly so that destructors for mex file objects
+      // are called, so that functions registered with mexAtExit are
+      // called.
+      clear_mex_functions ();
+
       command_editor::restore_terminal_state ();
 
-      // XXX FIXME XXX -- is this needed?  Can it cause any trouble?
+      // FIXME -- is this needed?  Can it cause any trouble?
       raw_mode (0);
 
       octave_history_write_timestamp ();
@@ -558,48 +652,104 @@ do_octave_atexit (void)
       flush_octave_stdout ();
 
       if (! quitting_gracefully && (interactive || forced_interactive))
-	std::cout << "\n";
+	{
+	  octave_stdout << "\n";
+
+	  // Yes, we want this to be separate from the call to
+	  // flush_octave_stdout above.
+
+	  flush_octave_stdout ();
+	}
     }
 }
 
-DEFUN (atexit, args, ,
+DEFUN (atexit, args, nargout,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} atexit (@var{fcn})\n\
 Register a function to be called when Octave exits.  For example,\n\
 \n\
 @example\n\
 @group\n\
-function print_fortune ()\n\
-  printf (\"\\n%s\\n\", system (\"fortune\"));\n\
-  fflush (stdout);\n\
+function bye_bye ()\n\
+  disp (\"Bye bye\");\n\
 endfunction\n\
-atexit (\"print_fortune\");\n\
+atexit (\"bye_bye\");\n\
 @end group\n\
 @end example\n\
 \n\
 @noindent\n\
-will print a message when Octave exits.\n\
+will print the message \"Bye bye\" when Octave exits.\n\
+\n\
+@deftypefnx {Built-in Function} {} atexit (@var{fcn}, @var{flag})\n\
+Register or unregister a function to be called when Octave exits,\n\
+depending on @var{flag}.  If @var{flag} is true, the function is\n\
+registered, if @var{flag} is false, it is unregistered.  For example,\n\
+after registering the function @code{bye_bye} as above,\n\
+\n\
+@example\n\
+atexit (\"bye_bye\", false);\n\
+@end example\n\
+\n\
+@noindent\n\
+will remove the function from the list and Octave will not call\n\
+the function @code{bye_by} when it exits.\n\
+\n\
+Note that @code{atexit} only removes the first occurrence of a function\n\
+from the list, so if a function was placed in the list multiple\n\
+times with @code{atexit}, it must also be removed from the list\n\
+multiple times.\n\
 @end deftypefn")
 {
   octave_value_list retval;
 
   int nargin = args.length ();
 
-  if (nargin == 1)
+  if (nargin == 1 || nargin == 2)
     {
-#if defined (HAVE_ATEXIT) || defined (HAVE_ON_EXIT)
       std::string arg = args(0).string_value ();
 
       if (! error_state)
-	octave_atexit_functions.push (arg);
+        {
+          bool add_mode = true;
+
+          if (nargin == 2)
+            {
+              add_mode = args(1).bool_value ();
+
+              if (error_state)
+                error ("atexit: second argument must be a logical value");
+            }
+
+          if (! error_state)
+	    {
+	      if (add_mode)
+		octave_atexit_functions.push_front (arg);
+	      else
+		{
+		  bool found = false;
+		  std::list<std::string>::iterator it;
+
+		  for (std::list<std::string>::iterator p = octave_atexit_functions.begin ();
+		       p != octave_atexit_functions.end (); p++)
+		    {
+		      if (*p == arg)
+			{
+			  octave_atexit_functions.erase (p);
+			  found = true;
+			  break;
+			}
+		    }
+
+		  if (nargout > 0)
+		    retval(0) = found;
+		}
+	    }
+	}
       else
-	error ("atexit: argument must be a string");
-#else
-      gripe_not_supported ("atexit");
-#endif
+        error ("atexit: argument must be a string");
     }
   else
-    print_usage ("atexit");
+    print_usage ();
 
   return retval;
 }
@@ -626,113 +776,119 @@ specified option.\n\
   static bool initialized = false;
   static Octave_map m;
 
-  static const char * const conf_info[] =
+  struct conf_info_struct
+  {
+    bool subst_home;
+    const char *key;
+    const char *val;
+  };
+
+  static const conf_info_struct conf_info[] =
     {
-      "ALL_CFLAGS", OCTAVE_CONF_ALL_CFLAGS,
-      "ALL_CXXFLAGS", OCTAVE_CONF_ALL_CXXFLAGS,
-      "ALL_FFLAGS", OCTAVE_CONF_ALL_FFLAGS,
-      "ALL_LDFLAGS", OCTAVE_CONF_ALL_LDFLAGS,
-      "AR", OCTAVE_CONF_AR,
-      "ARFLAGS", OCTAVE_CONF_ARFLAGS,
-      "BLAS_LIBS", OCTAVE_CONF_BLAS_LIBS,
-      "CC", OCTAVE_CONF_CC,
-      "CC_VERSION", OCTAVE_CONF_CC_VERSION,
-      "CFLAGS", OCTAVE_CONF_CFLAGS,
-      "CPICFLAG", OCTAVE_CONF_CPICFLAG,
-      "CPPFLAGS", OCTAVE_CONF_CPPFLAGS,
-      "CXX", OCTAVE_CONF_CXX,
-      "CXXCPP", OCTAVE_CONF_CXXCPP,
-      "CXXFLAGS", OCTAVE_CONF_CXXFLAGS,
-      "CXXPICFLAG", OCTAVE_CONF_CXXPICFLAG,
-      "CXX_VERSION", OCTAVE_CONF_CXX_VERSION,
-      "DEFAULT_PAGER", OCTAVE_DEFAULT_PAGER,
-      "DL_LD", OCTAVE_CONF_DL_LD,
-      "DL_LDFLAGS", OCTAVE_CONF_DL_LDFLAGS,
-      "DLFCN_INCFLAGS", OCTAVE_CONF_DLFCN_INCFLAGS,
-      "EXEEXT", OCTAVE_CONF_EXEEXT,
-      "F2C", OCTAVE_CONF_F2C,
-      "F2CFLAGS", OCTAVE_CONF_F2CFLAGS,
-      "F77", OCTAVE_CONF_F77,
-      "FC", OCTAVE_CONF_FC,
-      "FFLAGS", OCTAVE_CONF_FFLAGS,
-      "FFTW_LIBS", OCTAVE_CONF_FFTW_LIBS,
-      "FLIBS", OCTAVE_CONF_FLIBS,
-      "FPICFLAG", OCTAVE_CONF_FPICFLAG,
-      "GLPK_LIBS", OCTAVE_CONF_GLPK_LIBS,
-      "INCFLAGS", OCTAVE_CONF_INCFLAGS,
-      "LDFLAGS", OCTAVE_CONF_LDFLAGS,
-      "LD_CXX", OCTAVE_CONF_LD_CXX,
-      "LD_STATIC_FLAG", OCTAVE_CONF_LD_STATIC_FLAG,
-      "LEX", OCTAVE_CONF_LEX,
-      "LEXLIB", OCTAVE_CONF_LEXLIB,
-      "LFLAGS", OCTAVE_CONF_LFLAGS,
-      "LIBCRUFT", OCTAVE_CONF_LIBCRUFT,
-      "LIBDLFCN", OCTAVE_CONF_LIBDLFCN,
-      "LIBEXT", OCTAVE_CONF_LIBEXT,
-      "LIBFLAGS", OCTAVE_CONF_LIBFLAGS,
-      "LIBOCTAVE", OCTAVE_CONF_LIBOCTAVE,
-      "LIBOCTINTERP", OCTAVE_CONF_LIBOCTINTERP,
-      "LIBPLPLOT", OCTAVE_CONF_LIBPLPLOT,
-      "LIBREADLINE", OCTAVE_CONF_LIBREADLINE,
-      "LIBS", OCTAVE_CONF_LIBS,
-      "LN_S", OCTAVE_CONF_LN_S,
-      "MKOCTFILE_INCFLAGS", OCTAVE_CONF_MKOCTFILE_INCFLAGS,
-      "MKOCTFILE_LFLAGS", OCTAVE_CONF_MKOCTFILE_LFLAGS,
-      "MKOCTFILE_DL_LDFLAGS", OCTAVE_CONF_MKOCTFILE_DL_LDFLAGS,
-      "RANLIB", OCTAVE_CONF_RANLIB,
-      "RDYNAMIC_FLAG", OCTAVE_CONF_RDYNAMIC_FLAG,
-      "RLD_FLAG", OCTAVE_CONF_RLD_FLAG,
-      "RUNTEST", OCTAVE_CONF_RUNTEST,
-      "SED", OCTAVE_CONF_SED,
-      "SHARED_LIBS", OCTAVE_CONF_SHARED_LIBS,
-      "SHLEXT", OCTAVE_CONF_SHLEXT,
-      "SHLEXT_VER", OCTAVE_CONF_SHLEXT_VER,
-      "SH_LD", OCTAVE_CONF_SH_LD,
-      "SH_LDFLAGS", OCTAVE_CONF_SH_LDFLAGS,
-      "SONAME_FLAGS", OCTAVE_CONF_SONAME_FLAGS,
-      "STATIC_LIBS", OCTAVE_CONF_STATIC_LIBS,
-      "DEFS", OCTAVE_CONF_DEFS,
-      "UGLY_DEFS", OCTAVE_CONF_UGLY_DEFS,
-      "USE_64_BIT_IDX_T", OCTAVE_CONF_USE_64_BIT_IDX_T,
-      "ENABLE_DYNAMIC_LINKING", OCTAVE_CONF_ENABLE_DYNAMIC_LINKING,
-      "XTRA_CFLAGS", OCTAVE_CONF_XTRA_CFLAGS,
-      "XTRA_CXXFLAGS", OCTAVE_CONF_XTRA_CXXFLAGS,
-      "YACC", OCTAVE_CONF_YACC,
-      "YFLAGS", OCTAVE_CONF_YFLAGS,
-      "archlibdir", OCTAVE_ARCHLIBDIR,
-      "bindir", OCTAVE_BINDIR,
-      "canonical_host_type", OCTAVE_CANONICAL_HOST_TYPE,
-      "config_opts", OCTAVE_CONF_config_opts,
-      "datadir", OCTAVE_DATADIR,
-      "exec_prefix", OCTAVE_EXEC_PREFIX,
-      "fcnfiledir", OCTAVE_FCNFILEDIR,
-      "fcnfilepath", OCTAVE_FCNFILEPATH,
-      "imagedir", OCTAVE_IMAGEDIR,
-      "imagepath", OCTAVE_IMAGEPATH,
-      "includedir", OCTAVE_INCLUDEDIR,
-      "infodir", OCTAVE_INFODIR,
-      "infofile", OCTAVE_INFOFILE,
-      "libdir", OCTAVE_LIBDIR,
-      "libexecdir", OCTAVE_LIBEXECDIR,
-      "localarchlibdir", OCTAVE_LOCALARCHLIBDIR,
-      "localfcnfiledir", OCTAVE_LOCALFCNFILEDIR,
-      "localfcnfilepath", OCTAVE_LOCALFCNFILEPATH,
-      "localoctfiledir", OCTAVE_LOCALOCTFILEDIR,
-      "localoctfilepath", OCTAVE_LOCALOCTFILEPATH,
-      "localstartupfiledir", OCTAVE_LOCALSTARTUPFILEDIR,
-      "localverarchlibdir", OCTAVE_LOCALVERARCHLIBDIR,
-      "localverfcnfiledir", OCTAVE_LOCALVERFCNFILEDIR,
-      "localveroctfiledir", OCTAVE_LOCALVEROCTFILEDIR,
-      "man1dir", OCTAVE_MAN1DIR,
-      "man1ext", OCTAVE_MAN1EXT,
-      "mandir", OCTAVE_MANDIR,
-      "octfiledir", OCTAVE_OCTFILEDIR,
-      "octincludedir", OCTAVE_OCTINCLUDEDIR,
-      "octlibdir", OCTAVE_OCTLIBDIR,
-      "prefix", OCTAVE_PREFIX,
-      "startupfiledir", OCTAVE_STARTUPFILEDIR,
-      "version", OCTAVE_VERSION,
-      0, 0
+      { false, "ALL_CFLAGS", OCTAVE_CONF_ALL_CFLAGS },
+      { false, "ALL_CXXFLAGS", OCTAVE_CONF_ALL_CXXFLAGS },
+      { false, "ALL_FFLAGS", OCTAVE_CONF_ALL_FFLAGS },
+      { false, "ALL_LDFLAGS", OCTAVE_CONF_ALL_LDFLAGS },
+      { false, "AR", OCTAVE_CONF_AR },
+      { false, "ARFLAGS", OCTAVE_CONF_ARFLAGS },
+      { false, "BLAS_LIBS", OCTAVE_CONF_BLAS_LIBS },
+      { false, "CC", OCTAVE_CONF_CC },
+      { false, "CC_VERSION", OCTAVE_CONF_CC_VERSION },
+      { false, "CFLAGS", OCTAVE_CONF_CFLAGS },
+      { false, "CPICFLAG", OCTAVE_CONF_CPICFLAG },
+      { false, "CPPFLAGS", OCTAVE_CONF_CPPFLAGS },
+      { false, "CURL_LIBS", OCTAVE_CONF_CURL_LIBS },
+      { false, "CXX", OCTAVE_CONF_CXX },
+      { false, "CXXCPP", OCTAVE_CONF_CXXCPP },
+      { false, "CXXFLAGS", OCTAVE_CONF_CXXFLAGS },
+      { false, "CXXPICFLAG", OCTAVE_CONF_CXXPICFLAG },
+      { false, "CXX_VERSION", OCTAVE_CONF_CXX_VERSION },
+      { false, "DEFAULT_PAGER", OCTAVE_DEFAULT_PAGER },
+      { false, "DEFS", OCTAVE_CONF_DEFS },
+      { false, "DLFCN_INCFLAGS", OCTAVE_CONF_DLFCN_INCFLAGS },
+      { false, "DL_LD", OCTAVE_CONF_DL_LD },
+      { false, "DL_LDFLAGS", OCTAVE_CONF_DL_LDFLAGS },
+      { false, "ENABLE_DYNAMIC_LINKING", OCTAVE_CONF_ENABLE_DYNAMIC_LINKING },
+      { false, "EXEEXT", OCTAVE_CONF_EXEEXT },
+      { false, "F2C", OCTAVE_CONF_F2C },
+      { false, "F2CFLAGS", OCTAVE_CONF_F2CFLAGS },
+      { false, "F77", OCTAVE_CONF_F77 },
+      { false, "F77_FLOAT_STORE_FLAG", OCTAVE_CONF_F77_FLOAT_STORE_FLAG },
+      { false, "FC", OCTAVE_CONF_FC },
+      { false, "FFLAGS", OCTAVE_CONF_FFLAGS },
+      { false, "FFTW_LIBS", OCTAVE_CONF_FFTW_LIBS },
+      { false, "FLIBS", OCTAVE_CONF_FLIBS },
+      { false, "FPICFLAG", OCTAVE_CONF_FPICFLAG },
+      { false, "GLPK_LIBS", OCTAVE_CONF_GLPK_LIBS },
+      { false, "INCFLAGS", OCTAVE_CONF_INCFLAGS },
+      { false, "LDFLAGS", OCTAVE_CONF_LDFLAGS },
+      { false, "LD_CXX", OCTAVE_CONF_LD_CXX },
+      { false, "LD_STATIC_FLAG", OCTAVE_CONF_LD_STATIC_FLAG },
+      { false, "LEX", OCTAVE_CONF_LEX },
+      { false, "LEXLIB", OCTAVE_CONF_LEXLIB },
+      { false, "LFLAGS", OCTAVE_CONF_LFLAGS },
+      { false, "LIBCRUFT", OCTAVE_CONF_LIBCRUFT },
+      { false, "LIBDLFCN", OCTAVE_CONF_LIBDLFCN },
+      { false, "LIBEXT", OCTAVE_CONF_LIBEXT },
+      { false, "LIBFLAGS", OCTAVE_CONF_LIBFLAGS },
+      { false, "LIBOCTAVE", OCTAVE_CONF_LIBOCTAVE },
+      { false, "LIBOCTINTERP", OCTAVE_CONF_LIBOCTINTERP },
+      { false, "LIBREADLINE", OCTAVE_CONF_LIBREADLINE },
+      { false, "LIBS", OCTAVE_CONF_LIBS },
+      { false, "LN_S", OCTAVE_CONF_LN_S },
+      { false, "MKOCTFILE_DL_LDFLAGS", OCTAVE_CONF_MKOCTFILE_DL_LDFLAGS },
+      { false, "RANLIB", OCTAVE_CONF_RANLIB },
+      { false, "RDYNAMIC_FLAG", OCTAVE_CONF_RDYNAMIC_FLAG },
+      { false, "RLD_FLAG", OCTAVE_CONF_RLD_FLAG },
+      { false, "SED", OCTAVE_CONF_SED },
+      { false, "SHARED_LIBS", OCTAVE_CONF_SHARED_LIBS },
+      { false, "SHLEXT", OCTAVE_CONF_SHLEXT },
+      { false, "SHLEXT_VER", OCTAVE_CONF_SHLEXT_VER },
+      { false, "SH_LD", OCTAVE_CONF_SH_LD },
+      { false, "SH_LDFLAGS", OCTAVE_CONF_SH_LDFLAGS },
+      { false, "SONAME_FLAGS", OCTAVE_CONF_SONAME_FLAGS },
+      { false, "STATIC_LIBS", OCTAVE_CONF_STATIC_LIBS },
+      { false, "UGLY_DEFS", OCTAVE_CONF_UGLY_DEFS },
+      { false, "USE_64_BIT_IDX_T", OCTAVE_CONF_USE_64_BIT_IDX_T },
+      { false, "XTRA_CFLAGS", OCTAVE_CONF_XTRA_CFLAGS },
+      { false, "XTRA_CXXFLAGS", OCTAVE_CONF_XTRA_CXXFLAGS },
+      { false, "YACC", OCTAVE_CONF_YACC },
+      { false, "YFLAGS", OCTAVE_CONF_YFLAGS },
+      { false, "api_version", OCTAVE_API_VERSION },
+      { true, "archlibdir", OCTAVE_ARCHLIBDIR },
+      { true, "bindir", OCTAVE_BINDIR },
+      { false, "canonical_host_type", OCTAVE_CANONICAL_HOST_TYPE },
+      { false, "config_opts", OCTAVE_CONF_config_opts },
+      { true, "datadir", OCTAVE_DATADIR },
+      { true, "datarootdir", OCTAVE_DATAROOTDIR },
+      { true, "exec_prefix", OCTAVE_EXEC_PREFIX },
+      { true, "fcnfiledir", OCTAVE_FCNFILEDIR },
+      { true, "imagedir", OCTAVE_IMAGEDIR },
+      { true, "includedir", OCTAVE_INCLUDEDIR },
+      { true, "infodir", OCTAVE_INFODIR },
+      { true, "infofile", OCTAVE_INFOFILE },
+      { true, "libdir", OCTAVE_LIBDIR },
+      { true, "libexecdir", OCTAVE_LIBEXECDIR },
+      { true, "localapiarchlibdir", OCTAVE_LOCALAPIARCHLIBDIR },
+      { true, "localapifcnfiledir", OCTAVE_LOCALAPIFCNFILEDIR },
+      { true, "localapioctfiledir", OCTAVE_LOCALAPIOCTFILEDIR },
+      { true, "localarchlibdir", OCTAVE_LOCALARCHLIBDIR },
+      { true, "localfcnfiledir", OCTAVE_LOCALFCNFILEDIR },
+      { true, "localoctfiledir", OCTAVE_LOCALOCTFILEDIR },
+      { true, "localstartupfiledir", OCTAVE_LOCALSTARTUPFILEDIR },
+      { true, "localverarchlibdir", OCTAVE_LOCALVERARCHLIBDIR },
+      { true, "localverfcnfiledir", OCTAVE_LOCALVERFCNFILEDIR },
+      { true, "localveroctfiledir", OCTAVE_LOCALVEROCTFILEDIR },
+      { true, "man1dir", OCTAVE_MAN1DIR },
+      { false, "man1ext", OCTAVE_MAN1EXT },
+      { true, "mandir", OCTAVE_MANDIR },
+      { true, "octfiledir", OCTAVE_OCTFILEDIR },
+      { true, "octincludedir", OCTAVE_OCTINCLUDEDIR },
+      { true, "octlibdir", OCTAVE_OCTLIBDIR },
+      { true, "prefix", OCTAVE_PREFIX },
+      { true, "startupfiledir", OCTAVE_STARTUPFILEDIR },
+      { false, "version", OCTAVE_VERSION },
+      { false, 0, 0 }
     };
 
   if (! initialized)
@@ -753,15 +909,23 @@ specified option.\n\
 
       while (true)
 	{
-	  const char *key = conf_info[i++];
+	  const conf_info_struct& elt = conf_info[i++];
+
+	  const char *key = elt.key;
 
 	  if (key)
-	    m.assign (key, octave_value (conf_info[i++]));
+	    {
+	      if (elt.subst_home)
+		m.assign (key, octave_value (subst_octave_home (elt.val)));
+	      else
+		m.assign (key, octave_value (elt.val));
+	    }
 	  else
 	    break;
 	}
 
       bool unix_system = true;
+      bool mac_system = false;
       bool windows_system = false;
 
 #if defined (WIN32)
@@ -771,7 +935,12 @@ specified option.\n\
 #endif
 #endif
 
+#if defined (__APPLE__) && defined (__MACH__)
+      mac_system = true;
+#endif
+
       m.assign ("unix", octave_value (unix_system));
+      m.assign ("mac", octave_value (mac_system));
       m.assign ("windows", octave_value (windows_system));
 
       initialized = true;
@@ -796,7 +965,7 @@ specified option.\n\
   else if (nargin == 0)
     retval = m;
   else
-    print_usage ("octave_config_info");
+    print_usage ();
 
   return retval;
 }
@@ -824,7 +993,7 @@ __builtin_new (size_t sz)
     }
 
   if (debug_new_delete)
-    std::cout << "__builtin_new: " << p << std::endl;
+    std::cerr << "__builtin_new: " << p << std::endl;
 
   return p;
 }
@@ -833,7 +1002,7 @@ void
 __builtin_delete (void *ptr)
 {
   if (debug_new_delete)
-    std::cout << "__builtin_delete: " << ptr << std::endl;
+    std::cerr << "__builtin_delete: " << ptr << std::endl;
 
   if (ptr)
     free (ptr);
@@ -841,57 +1010,6 @@ __builtin_delete (void *ptr)
 
 #endif
 
-void
-symbols_of_toplev (void)
-{
-  DEFCONST (argv, ,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Constant} argv\n\
-The command line arguments passed to Octave are available in this\n\
-variable.  For example, if you invoked Octave using the command\n\
-\n\
-@example\n\
-octave --no-line-editing --silent\n\
-@end example\n\
-\n\
-@noindent\n\
-@code{argv} would be a cell array of strings with the elements\n\
-@code{--no-line-editing} and @code{--silent}.\n\
-\n\
-If you write an executable Octave script, @code{argv} will contain the\n\
-list of arguments passed to the script.  @xref{Executable Octave Programs},\n\
-for an example of how to create an executable Octave script.\n\
-@end defvr");
-
-  DEFCONST (program_invocation_name,
-	    octave_env::get_program_invocation_name (),
-    "-*- texinfo -*-\n\
-@defvr {Built-in Constant} program_invocation_name\n\
-@defvrx {Built-in Constant} program_name\n\
-When Octave starts, the value of the built-in variable\n\
-@code{program_invocation_name} is automatically set to the name that was\n\
-typed at the shell prompt to run Octave, and the value of\n\
-@code{program_name} is automatically set to the final component of\n\
-@code{program_invocation_name}.  For example, if you typed\n\
-@samp{@value{OCTAVEHOME}/bin/octave} to start Octave,\n\
-@code{program_invocation_name} would have the value\n\
-@code{\"@value{OCTAVEHOME}/bin/octave\"}, and @code{program_name} would\n\
-have the value @code{\"octave\"}.\n\
-\n\
-If executing a script from the command line (e.g., @code{octave foo.m})\n\
-or using an executable Octave script, the program name is set to the\n\
-name of the script.  @xref{Executable Octave Programs}, for an example of\n\
-how to create an executable Octave script.\n\
-@end defvr");
-
-  DEFCONST (program_name, octave_env::get_program_name (),
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} program_name\n\
-See: program_invocation_name.\n\
-@end defvr");
-
-}
-
 /*
 ;;; Local Variables: ***
 ;;; mode: C++ ***
